from __future__ import annotations

import json
import logging
import os.path
import re
import uuid
from collections import namedtuple
from concurrent import futures
from io import StringIO
from json import JSONDecodeError
from os import path
from pathlib import Path
from time import sleep
from types import MethodType
from typing import List, Dict, TYPE_CHECKING, Any, Set, cast, Optional, Union
from urllib.parse import urlparse

import boto3
import dpath
import urllib3
from botocore.config import Config
from botocore.exceptions import ClientError
from cachetools import cached, TTLCache
from colorama import Style
from termcolor import colored
from tqdm import trange
from urllib3.exceptions import HTTPError, MaxRetryError

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.platform_errors import BridgecrewAuthError
from checkov.common.bridgecrew.platform_key import read_key
from checkov.common.bridgecrew.run_metadata.registry import registry
from checkov.common.bridgecrew.wrapper import CDK_FRAMEWORK_PREFIX, persist_assets_results, reduce_scan_reports, persist_checks_results, \
    enrich_and_persist_checks_metadata, checkov_results_prefix, persist_run_metadata, _put_json_object, \
    persist_logs_stream, persist_graphs, persist_resource_subgraph_maps, persist_reachability_results
from checkov.common.models.consts import SAST_SUPPORTED_FILE_EXTENSIONS, SUPPORTED_FILE_EXTENSIONS, SUPPORTED_FILES, SCANNABLE_PACKAGE_FILES
from checkov.common.runners.base_runner import filter_ignored_paths
from checkov.common.sast.consts import SastLanguages
from checkov.common.typing import _CicdDetails, LibraryGraph
from checkov.common.util.consts import PRISMA_PLATFORM, BRIDGECREW_PLATFORM, CHECKOV_RUN_SCA_PACKAGE_SCAN_V2
from checkov.common.util.data_structures_utils import merge_dicts
from checkov.common.util.dockerfile import is_dockerfile
from checkov.common.util.http_utils import (
    normalize_prisma_url,
    get_auth_header,
    get_default_get_headers,
    get_user_agent_header,
    get_default_post_headers,
    get_prisma_get_headers,
    get_prisma_auth_header,
    get_auth_error_message,
    normalize_bc_url,
    REQUEST_CONNECT_TIMEOUT,
    REQUEST_READ_TIMEOUT,
    REQUEST_RETRIES,
)
from checkov.common.util.type_forcers import convert_prisma_policy_filter_to_dict, convert_str_to_bool
from checkov.version import version as checkov_version

if TYPE_CHECKING:
    import argparse
    from checkov.common.bridgecrew.bc_source import SourceType
    from checkov.common.output.report import Report
    from checkov.secrets.coordinator import EnrichedSecret
    from mypy_boto3_s3.client import S3Client
    from typing_extensions import TypeGuard
    from checkov.common.sast.report_types import Match

SLEEP_SECONDS = 1

EMAIL_PATTERN = re.compile(r"[^@]+@[^@]+\.[^@]+")
UUID_V4_PATTERN = re.compile(r"^[0-9a-f]{8}\b-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-\b[0-9a-f]{12}$")
# found at https://regexland.com/base64/
BASE64_PATTERN = re.compile(r"^(?:[A-Za-z\d+/]{4})*(?:[A-Za-z\d+/]{3}=|[A-Za-z\d+/]{2}==)?$")
REPO_PATH_PATTERN = re.compile(r'checkov/(.*?)/src')

ACCOUNT_CREATION_TIME = 180  # in seconds

UNAUTHORIZED_MESSAGE = 'User is not authorized to access this resource with an explicit deny'
ASSUME_ROLE_UNUATHORIZED_MESSAGE = 'is not authorized to perform: sts:AssumeRole'

FileToPersist = namedtuple('FileToPersist', 'full_file_path s3_file_key')

DEFAULT_REGION = "us-west-2"
GOV_CLOUD_REGION = 'us-gov-west-1'
PRISMA_GOV_API_URL = 'https://api.gov.prismacloud.io'
MAX_RETRIES = 40

CI_METADATA_EXTRACTOR = registry.get_extractor()


class BcPlatformIntegration:
    def __init__(self) -> None:
        self.clean()

    def clean(self) -> None:
        self.bc_api_key = read_key()
        self.s3_client: S3Client | None = None
        self.bucket: str | None = None
        self.credentials: dict[str, str] | None = None
        self.repo_path: str | None = None
        self.support_bucket: str | None = None
        self.support_repo_path: str | None = None
        self.repo_id: str | None = None
        self.repo_branch: str | None = None
        self.skip_fixes = False  # even though we removed the CLI flag, this gets set so we know whether this is a fix run (IDE) or not (normal CLI)
        self.skip_download = False
        self.source_id: str | None = None
        self.bc_source: SourceType | None = None
        self.bc_source_version: str | None = None
        self.timestamp: str | None = None
        self.scan_reports: list[Report] = []
        self.bc_api_url = normalize_bc_url(os.getenv('BC_API_URL', "https://www.bridgecrew.cloud"))
        self.prisma_api_url = normalize_prisma_url(os.getenv("PRISMA_API_URL"))
        self.prisma_policies_url: str | None = None
        self.prisma_policy_filters_url: str | None = None
        self.setup_api_urls()
        self.customer_run_config_response = None
        self.runtime_run_config_response = None
        self.prisma_policies_response = None
        self.public_metadata_response = None
        self.use_s3_integration = False
        self.s3_setup_failed = False
        self.platform_integration_configured = False
        self.http: urllib3.PoolManager | urllib3.ProxyManager | None = None
        self.http_timeout = urllib3.Timeout(connect=REQUEST_CONNECT_TIMEOUT, read=REQUEST_READ_TIMEOUT)
        self.http_retry = urllib3.Retry(REQUEST_RETRIES, redirect=3)
        self.bc_skip_mapping = False
        self.cicd_details: _CicdDetails = {}
        self.support_flag_enabled = False
        self.enable_persist_graphs = convert_str_to_bool(os.getenv('BC_ENABLE_PERSIST_GRAPHS', 'True'))
        self.persist_graphs_timeout = int(os.getenv('BC_PERSIST_GRAPHS_TIMEOUT', 60))
        self.ca_certificate: str | None = None
        self.no_cert_verify: bool = False
        self.on_prem: bool = False
        self.daemon_process = False  # set to 'True' when running in multiprocessing 'spawn' mode
        self.scan_dir: List[str] = []
        self.scan_file: List[str] = []

    def init_instance(self, platform_integration_data: dict[str, Any]) -> None:
        """This is mainly used for recreating the instance without interacting with the platform again"""

        self.daemon_process = True

        self.bc_api_url = platform_integration_data["bc_api_url"]
        self.bc_api_key = platform_integration_data["bc_api_key"]
        self.bc_source = platform_integration_data["bc_source"]
        self.bc_source_version = platform_integration_data["bc_source_version"]
        self.bucket = platform_integration_data["bucket"]
        self.cicd_details = platform_integration_data["cicd_details"]
        self.credentials = platform_integration_data["credentials"]
        self.platform_integration_configured = platform_integration_data["platform_integration_configured"]
        self.prisma_api_url = platform_integration_data["prisma_api_url"]
        self.repo_branch = platform_integration_data["repo_branch"]
        self.repo_id = platform_integration_data["repo_id"]
        self.repo_path = platform_integration_data["repo_path"]
        self.skip_fixes = platform_integration_data["skip_fixes"]
        self.timestamp = platform_integration_data["timestamp"]
        self.use_s3_integration = platform_integration_data["use_s3_integration"]
        self.setup_api_urls()
        # 'mypy' doesn't like, when you try to override an instance method
        self.get_auth_token = MethodType(lambda _=None: platform_integration_data["get_auth_token"], self)  # type:ignore[method-assign]

    def generate_instance_data(self) -> dict[str, Any]:
        """This output is used to re-initialize the instance and should be kept in sync with 'init_instance()'"""

        return {
            # 'api_url' will be set by invoking 'setup_api_urls()'
            "bc_api_url": self.bc_api_url,
            "bc_api_key": self.bc_api_key,
            "bc_source": self.bc_source,
            "bc_source_version": self.bc_source_version,
            "bucket": self.bucket,
            "cicd_details": self.cicd_details,
            "credentials": self.credentials,
            "platform_integration_configured": self.platform_integration_configured,
            "prisma_api_url": self.prisma_api_url,
            "repo_branch": self.repo_branch,
            "repo_id": self.repo_id,
            "repo_path": self.repo_path,
            "skip_fixes": self.skip_fixes,
            "timestamp": self.timestamp,
            "use_s3_integration": self.use_s3_integration,
            # will be overriden with a simple lambda expression
            "get_auth_token": self.get_auth_token() if self.bc_api_key else ""
        }

    def set_bc_api_url(self, new_url: str) -> None:
        self.bc_api_url = normalize_bc_url(new_url)
        self.setup_api_urls()

    def setup_api_urls(self) -> None:
        """
        API URLs vary depending upon whether the platform is Bridgecrew or Prisma Cloud.
        Bridgecrew has one default that can be used when initializing the class,
        but Prisma Cloud requires resetting them in setup_bridgecrew_credentials,
        which is where command-line parameters are first made available.
        """
        if self.prisma_api_url:
            self.api_url = f"{self.prisma_api_url}/bridgecrew"
            self.prisma_policies_url = f"{self.prisma_api_url}/v2/policy"
            self.prisma_policy_filters_url = f"{self.prisma_api_url}/filter/policy/suggest"
        else:
            self.api_url = self.bc_api_url
        self.guidelines_api_url = f"{self.api_url}/api/v2/guidelines"
        self.guidelines_api_url_backoff = f"{self.api_url}/api/v1/guidelines"

        self.integrations_api_url = f"{self.api_url}/api/v1/integrations/types/checkov"
        self.platform_run_config_url = f"{self.api_url}/api/v2/checkov/runConfiguration"
        self.reachability_run_config_url = f"{self.api_url}/api/v2/checkov/reachabilityRunConfiguration"
        self.runtime_run_config_url = f"{self.api_url}/api/v1/runtime-images/repositories"

    def is_prisma_integration(self) -> bool:
        if self.bc_api_key and not self.is_bc_token(self.bc_api_key):
            return True
        return False

    @staticmethod
    def is_token_valid(token: str) -> bool:
        parts = token.split('::')
        parts_len = len(parts)
        if parts_len == 1:
            valid = BcPlatformIntegration.is_bc_token(token)
            # TODO: add it back at a later time
            # if valid:
            #     print(
            #         "We're glad you're using Checkov with Bridgecrew!\n"
            #         "Bridgecrew has been fully integrated into Prisma Cloud with a powerful code to cloud experience.\n"
            #         "As a part of the transition, we will be shutting down Bridgecrew standalone edition at the end of 2023 (https://www.paloaltonetworks.com/services/support/end-of-life-announcements).\n"
            #         "Please upgrade to Prisma Cloud Enterprise Edition before the end of the year.\n"
            #     )

            return valid
        elif parts_len == 2:
            # A Prisma access key is a UUID, same as a BC API key
            if BcPlatformIntegration.is_bc_token(parts[0]) and parts[1] and BASE64_PATTERN.match(parts[1]) is not None:
                return True
            return False
        else:
            return False

    @staticmethod
    def is_bc_token(token: str | None) -> TypeGuard[str]:
        if not token:
            return False

        return re.match(UUID_V4_PATTERN, token) is not None

    @cached(TTLCache(maxsize=1, ttl=540))
    def get_auth_token(self) -> str:
        if self.is_bc_token(self.bc_api_key):
            return self.bc_api_key
        # A Prisma Cloud Access Key was specified as the Bridgecrew token.
        if not self.prisma_api_url:
            raise ValueError("A Prisma Cloud token was set, but no Prisma Cloud API URL was set")
        if not self.bc_api_key:
            # should usually not happen
            raise ValueError("A Prisma Cloud or Birdgecrew token was not set")
        if '::' not in self.bc_api_key:
            raise ValueError(
                "A Prisma Cloud token was set, but the token is not in the correct format: <access_key_id>::<secret_key>")
        if not self.http:
            raise AttributeError("HTTP manager was not correctly created")
        username, password = self.bc_api_key.split('::')
        request = self.http.request("POST", f"{self.prisma_api_url}/login",  # type:ignore[no-untyped-call]
                                    body=json.dumps({"username": username, "password": password}),
                                    headers=merge_dicts({"Content-Type": "application/json"}, get_user_agent_header()))
        if request.status == 401:
            logging.error(f'Received 401 response from Prisma /login endpoint: {request.data.decode("utf8")}')
            raise BridgecrewAuthError()
        elif request.status == 403:
            logging.error('Received 403 (Forbidden) response from Prisma /login endpoint')
            raise BridgecrewAuthError()
        token: str = json.loads(request.data.decode("utf8"))['token']
        return token

    def setup_http_manager(self, ca_certificate: str | None = None, no_cert_verify: bool = False) -> None:
        """
        bridgecrew uses both the urllib3 and requests libraries, while checkov uses the requests library.
        :param ca_certificate: an optional CA bundle to be used by both libraries.
        :param no_cert_verify: whether to skip SSL cert verification
        """
        self.ca_certificate = ca_certificate
        self.no_cert_verify = no_cert_verify

        ca_certificate = ca_certificate or os.getenv('BC_CA_BUNDLE')
        cert_reqs: str | None

        if self.http:
            return
        if ca_certificate:
            os.environ['REQUESTS_CA_BUNDLE'] = ca_certificate
            cert_reqs = 'CERT_NONE' if no_cert_verify else 'REQUIRED'
            logging.debug(f'Using CA cert {ca_certificate} and cert_reqs {cert_reqs}')
            try:
                parsed_url = urllib3.util.parse_url(os.environ['https_proxy'])
                self.http = urllib3.ProxyManager(
                    os.environ['https_proxy'],
                    cert_reqs=cert_reqs,
                    ca_certs=ca_certificate,
                    proxy_headers=urllib3.make_headers(proxy_basic_auth=parsed_url.auth),  # type:ignore[no-untyped-call]
                    timeout=self.http_timeout,
                    retries=self.http_retry,
                )
            except KeyError:
                self.http = urllib3.PoolManager(
                    cert_reqs=cert_reqs,
                    ca_certs=ca_certificate,
                    timeout=self.http_timeout,
                    retries=self.http_retry,
                )
        else:
            cert_reqs = 'CERT_NONE' if no_cert_verify else None
            logging.debug(f'Using cert_reqs {cert_reqs}')
            try:
                parsed_url = urllib3.util.parse_url(os.environ['https_proxy'])
                self.http = urllib3.ProxyManager(
                    os.environ['https_proxy'],
                    cert_reqs=cert_reqs,
                    proxy_headers=urllib3.make_headers(proxy_basic_auth=parsed_url.auth),  # type:ignore[no-untyped-call]
                    timeout=self.http_timeout,
                    retries=self.http_retry,
                )
            except KeyError:
                self.http = urllib3.PoolManager(
                    cert_reqs=cert_reqs,
                    timeout=self.http_timeout,
                    retries=self.http_retry,
                )
        logging.debug('Successfully set up HTTP manager')

    def setup_bridgecrew_credentials(
        self,
        repo_id: str,
        skip_download: bool = False,
        source: SourceType | None = None,
        skip_fixes: bool = False,
        source_version: str | None = None,
        repo_branch: str | None = None,
        prisma_api_url: str | None = None,
    ) -> None:
        """
        Setup credentials against Bridgecrew's platform.
        :param repo_id: Identity string of the scanned repository, of the form <repo_owner>/<repo_name>
        :param skip_download: whether to skip downloading data (guidelines, custom policies, etc) from the platform
        :param source:
        :param prisma_api_url: optional URL for the Prisma Cloud platform, requires a Prisma Cloud Access Key as bc_api_key
        """
        self.repo_id = repo_id
        self.repo_branch = repo_branch
        self.skip_fixes = skip_fixes
        self.skip_download = skip_download
        self.bc_source = source
        self.bc_source_version = source_version

        if prisma_api_url:
            self.prisma_api_url = normalize_prisma_url(prisma_api_url)
            self.setup_api_urls()
            logging.info(f'Using Prisma API URL: {self.prisma_api_url}')

        if self.bc_source and self.bc_source.upload_results:
            self.set_s3_integration()

        self.platform_integration_configured = True

    def set_s3_integration(self) -> None:
        try:
            self.skip_fixes = True  # no need to run fixes on CI integration
            repo_full_path, support_path, response = self.get_s3_role(self.repo_id)  # type: ignore
            if not repo_full_path:  # happens if the setup fails with something other than an auth error - we continue locally
                return

            self.bucket, self.repo_path = repo_full_path.split("/", 1)

            self.timestamp = self.repo_path.split("/")[-2]
            self.credentials = cast("dict[str, str]", response["creds"])

            self.set_s3_client()

            if self.support_flag_enabled:
                self.support_bucket, self.support_repo_path = cast(str, support_path).split("/", 1)

            self.use_s3_integration = True
            self.platform_integration_configured = True
        except MaxRetryError:
            logging.error("An SSL error occurred connecting to the platform. If you are on a VPN, please try "
                          "disabling it and re-running the command.", exc_info=True)
            raise
        except HTTPError:
            logging.error("Failed to get customer assumed role", exc_info=True)
            raise
        except JSONDecodeError:
            logging.error(f"Response of {self.integrations_api_url} is not a valid JSON", exc_info=True)
            raise
        except BridgecrewAuthError:
            logging.error("Received an error response during authentication")
            raise

    def set_s3_client(self) -> None:
        if not self.credentials:
            raise ValueError("Credentials for client are not set")

        region = DEFAULT_REGION
        use_accelerate_endpoint = True
        if self.prisma_api_url == PRISMA_GOV_API_URL:
            region = GOV_CLOUD_REGION
            use_accelerate_endpoint = False

        try:
            config = Config(
                s3={
                    "use_accelerate_endpoint": use_accelerate_endpoint,
                }
            )
            self.s3_client = boto3.client(
                "s3",
                aws_access_key_id=self.credentials["AccessKeyId"],
                aws_secret_access_key=self.credentials["SecretAccessKey"],
                aws_session_token=self.credentials["SessionToken"],
                region_name=region,
                config=config,
            )
        except ClientError:
            logging.error(f"Failed to initiate client with credentials {self.credentials}", exc_info=True)
            raise

    def get_s3_role(self, repo_id: str) -> tuple[str, str, dict[str, Any]] | tuple[None, None, dict[str, Any]]:
        token = self.get_auth_token()

        if not self.http:
            raise AttributeError("HTTP manager was not correctly created")

        tries = 0
        response = self._get_s3_creds(repo_id, token)
        while ('Message' in response or 'message' in response):
            if response.get('Message') and response['Message'] == UNAUTHORIZED_MESSAGE:
                raise BridgecrewAuthError()
            elif response.get('message') and ASSUME_ROLE_UNUATHORIZED_MESSAGE in response['message']:
                raise BridgecrewAuthError(
                    "Checkov got an unexpected authorization error that may not be due to your credentials. Please contact support.")
            elif response.get('message') and "cannot be found" in response['message']:
                self.loading_output("creating role")
                response = self._get_s3_creds(repo_id, token)
            else:
                if tries < 3:
                    tries += 1
                    response = self._get_s3_creds(repo_id, token)
                else:
                    logging.error('Checkov got an unexpected error that may be due to backend issues. The scan will continue, '
                                  'but results will not be sent to the platform. Please contact support for assistance.')
                    logging.error(f'Error from platform: {response.get("message") or response.get("Message")}')
                    self.s3_setup_failed = True
                    return None, None, response
        repo_full_path = response["path"]
        support_path = response.get("supportPath")
        return repo_full_path, support_path, response

    def _get_s3_creds(self, repo_id: str, token: str) -> dict[str, Any]:
        logging.debug(f'Getting S3 upload credentials from {self.integrations_api_url}')
        request = self.http.request("POST", self.integrations_api_url,  # type:ignore[union-attr]
                                    body=json.dumps({"repoId": repo_id, "support": self.support_flag_enabled}),
                                    headers=merge_dicts({"Authorization": token, "Content-Type": "application/json"},
                                                        get_user_agent_header()))
        logging.debug(f'Request ID: {request.headers.get("x-amzn-requestid")}')
        logging.debug(f'Trace ID: {request.headers.get("x-amzn-trace-id")}')
        if request.status == 403:
            error_message = get_auth_error_message(request.status, self.is_prisma_integration(), True)
            raise BridgecrewAuthError(error_message)
        response: dict[str, Any] = json.loads(request.data.decode("utf8"))
        return response

    def is_integration_configured(self) -> bool:
        """
        Checks if Bridgecrew integration is fully configured based in input params.
        :return: True if the integration is configured, False otherwise
        """
        return self.platform_integration_configured

    def persist_repository(
        self,
        root_dir: str | Path,
        files: list[str] | None = None,
        excluded_paths: list[str] | None = None,
        included_paths: list[str] | None = None,
        sast_languages: Set[SastLanguages] | None = None
    ) -> None:
        """
        Persist the repository found on root_dir path to Bridgecrew's platform. If --file flag is used, only files
        that are specified will be persisted.
        :param files: Absolute path of the files passed in the --file flag.
        :param root_dir: Absolute path of the directory containing the repository root level.
        :param excluded_paths: Paths to exclude from persist process
        :param included_paths: Paths to exclude from persist process
        """
        excluded_paths = excluded_paths if excluded_paths is not None else []

        if not self.use_s3_integration or self.s3_setup_failed:
            return
        files_to_persist: List[FileToPersist] = []
        if files:
            for f in files:
                f_name = os.path.basename(f)
                _, file_extension = os.path.splitext(f)
                if CHECKOV_RUN_SCA_PACKAGE_SCAN_V2 and file_extension in SCANNABLE_PACKAGE_FILES:
                    continue
                if file_extension in SUPPORTED_FILE_EXTENSIONS or f_name in SUPPORTED_FILES:
                    files_to_persist.append(FileToPersist(f, os.path.relpath(f, root_dir)))
                if sast_languages:
                    for framwork in sast_languages:
                        if file_extension in SAST_SUPPORTED_FILE_EXTENSIONS[framwork]:
                            files_to_persist.append(FileToPersist(f, os.path.relpath(f, root_dir)))
                            break

        else:
            for root_path, d_names, f_names in os.walk(root_dir):
                # self.excluded_paths only contains the config fetched from the platform.
                # but here we expect the list from runner_registry as well (which includes self.excluded_paths).
                filter_ignored_paths(root_path, d_names, excluded_paths, included_paths=included_paths)
                filter_ignored_paths(root_path, f_names, excluded_paths)
                for file_path in f_names:
                    _, file_extension = os.path.splitext(file_path)
                    if CHECKOV_RUN_SCA_PACKAGE_SCAN_V2 and file_extension in SCANNABLE_PACKAGE_FILES:
                        continue
                    full_file_path = os.path.join(root_path, file_path)
                    relative_file_path = os.path.relpath(full_file_path, root_dir)
                    if file_extension in SUPPORTED_FILE_EXTENSIONS or file_path in SUPPORTED_FILES or is_dockerfile(file_path):
                        files_to_persist.append(FileToPersist(full_file_path, relative_file_path))
                    if sast_languages:
                        for framwork in sast_languages:
                            if file_extension in SAST_SUPPORTED_FILE_EXTENSIONS[framwork]:
                                files_to_persist.append(FileToPersist(full_file_path, relative_file_path))
                                break

        self.persist_files(files_to_persist)

    def persist_git_configuration(self, root_dir: str | Path, git_config_folders: list[str]) -> None:
        if not self.use_s3_integration or self.s3_setup_failed:
            return
        files_to_persist: list[FileToPersist] = []

        for git_config_folder in git_config_folders:
            if not os.path.isdir(git_config_folder):
                continue
            if not len(os.listdir(git_config_folder)):
                continue

            for root_path, _, f_names in os.walk(git_config_folder):
                for file_path in f_names:
                    _, file_extension = os.path.splitext(file_path)
                    if file_extension in SUPPORTED_FILE_EXTENSIONS:
                        full_file_path = os.path.join(root_path, file_path)
                        relative_file_path = os.path.relpath(full_file_path, root_dir)
                        files_to_persist.append(FileToPersist(full_file_path, relative_file_path))

        self.persist_files(files_to_persist)

    def adjust_sast_match_location_path(self, match: Match) -> None:
        for dir in self.scan_dir:
            if not match.location.path.startswith(dir):
                continue
            match.location.path = match.location.path.replace(dir, self.repo_path)  # type: ignore
            return
        for file in self.scan_file:
            if match.location.path != file:
                continue
            file_dir = '/'.join(match.location.path.split('/')[0:-1])
            match.location.path = match.location.path.replace(file_dir, self.repo_path)  # type: ignore
            return

    @staticmethod
    def _delete_code_block_from_sast_report(report: Dict[str, Any]) -> None:
        if isinstance(report, dict):
            for key, value in report.items():
                if key == 'code_block':
                    report[key] = ''
                BcPlatformIntegration._delete_code_block_from_sast_report(value)
        if isinstance(report, list):
            for item in report:
                BcPlatformIntegration._delete_code_block_from_sast_report(item)

    @staticmethod
    def save_sast_report_locally(sast_scan_reports: Dict[str, Dict[str, Any]]) -> None:
        for lang, report in sast_scan_reports.items():
            filename = f'{lang}_report.json'
            with open(f"/tmp/{filename}", 'w') as f:  # nosec
                f.write(json.dumps(report))

    def persist_sast_scan_results(self, reports: List[Report]) -> None:
        sast_scan_reports = {}
        for report in reports:
            if not report.check_type.startswith('sast'):
                continue
            if not report.sast_report:  # type: ignore
                continue
            for _, match_by_check in report.sast_report.rule_match.items():  # type: ignore
                for _, match in match_by_check.items():
                    for m in match.matches:
                        self.adjust_sast_match_location_path(m)
                sast_scan_reports[report.check_type] = report.sast_report.model_dump(mode='json')  # type: ignore
            if self.on_prem:
                BcPlatformIntegration._delete_code_block_from_sast_report(sast_scan_reports)

        if os.getenv('SAVE_SAST_REPORT_LOCALLY'):
            self.save_sast_report_locally(sast_scan_reports)

        persist_checks_results(sast_scan_reports, self.s3_client, self.bucket, self.repo_path)  # type: ignore

    def persist_cdk_scan_results(self, reports: List[Report]) -> None:
        cdk_scan_reports = {}
        for report in reports:
            if not report.check_type.startswith(CDK_FRAMEWORK_PREFIX):
                continue
            if not report.cdk_report:  # type: ignore
                continue
            for match_by_check in report.cdk_report.rule_match.values():  # type: ignore
                for _, match in match_by_check.items():
                    for m in match.matches:
                        self.adjust_sast_match_location_path(m)
                cdk_scan_reports[report.check_type] = report.cdk_report.model_dump(mode='json')  # type: ignore
            if self.on_prem:
                BcPlatformIntegration._delete_code_block_from_sast_report(cdk_scan_reports)

        # In case we dont have sast report - create empty one
        sast_reports = {}
        for check_type, report in cdk_scan_reports.items():
            lang = check_type.split('_')[1]
            found_sast_report = False
            for report in reports:
                if report.check_type == f'sast_{lang}':
                    found_sast_report = True
            if not found_sast_report:
                sast_reports[f'sast_{lang}'] = report.empty_sast_report.model_dump(mode='json')  # type: ignore

        persist_checks_results(sast_reports, self.s3_client, self.bucket, self.repo_path)  # type: ignore
        persist_checks_results(cdk_scan_reports, self.s3_client, self.bucket, self.repo_path)  # type: ignore

    def persist_scan_results(self, scan_reports: list[Report]) -> None:
        """
        Persist checkov's scan result into bridgecrew's platform.
        :param scan_reports: List of checkov scan reports
        """
        if not self.use_s3_integration or not self.s3_client or self.s3_setup_failed:
            return
        if not self.bucket or not self.repo_path:
            logging.error(f"Something went wrong: bucket {self.bucket}, repo path {self.repo_path}")
            return

        # just process reports with actual results in it
        self.scan_reports = [scan_report for scan_report in scan_reports if not scan_report.is_empty(full=True)]

        reduced_scan_reports = reduce_scan_reports(self.scan_reports, self.on_prem)
        checks_metadata_paths = enrich_and_persist_checks_metadata(self.scan_reports, self.s3_client, self.bucket,
                                                                   self.repo_path, self.on_prem)
        dpath.merge(reduced_scan_reports, checks_metadata_paths)
        persist_checks_results(reduced_scan_reports, self.s3_client, self.bucket, self.repo_path)

    async def persist_reachability_alias_mapping(self, alias_mapping: Dict[str, Any]) -> None:
        if not self.use_s3_integration or not self.s3_client or self.s3_setup_failed:
            return
        if not self.bucket or not self.repo_path:
            logging.error(f"Something went wrong: bucket {self.bucket}, repo path {self.repo_path}")
            return

        s3_path = f'{self.repo_path}/alias_mapping.json'
        _put_json_object(self.s3_client, alias_mapping, self.bucket, s3_path)

    def persist_assets_scan_results(self, assets_report: Optional[Dict[str, Any]]) -> None:
        if not assets_report:
            return
        for lang, assets in assets_report['imports'].items():
            new_report = {'imports': {lang.value: assets}}
            persist_assets_results(f'sast_{lang.value}', new_report, self.s3_client, self.bucket, self.repo_path)

    def persist_reachability_scan_results(self, reachability_report: Optional[Dict[str, Any]]) -> None:
        if not reachability_report:
            return
        for lang, report in reachability_report.items():
            persist_reachability_results(f'sast_{lang}', {lang: report}, self.s3_client, self.bucket, self.repo_path)

    def persist_image_scan_results(self, report: dict[str, Any] | None, file_path: str, image_name: str, branch: str) -> None:
        if not self.s3_client:
            logging.error("S3 upload was not correctly initialized")
            return
        if not self.bucket or not self.repo_path:
            logging.error("Bucket or repo_path was not set")
            return

        repo_path_without_src = os.path.dirname(self.repo_path)
        target_report_path = f'{repo_path_without_src}/{checkov_results_prefix}/{CheckType.SCA_IMAGE}/raw_results.json'
        to_upload = {"report": report, "file_path": file_path, "image_name": image_name, "branch": branch}
        _put_json_object(self.s3_client, to_upload, self.bucket, target_report_path)

    def persist_enriched_secrets(self, enriched_secrets: list[EnrichedSecret]) -> str | None:
        if not enriched_secrets or not self.repo_path or not self.bucket:
            logging.debug(f'One of enriched secrets, repo path, or bucket are empty, aborting. values:'
                          f'enriched_secrets={"Valid" if enriched_secrets else "Empty"},'
                          f' repo_path={self.repo_path}, bucket={self.bucket}')
            return None

        if not bc_integration.bc_api_key or not os.getenv("CKV_VALIDATE_SECRETS"):
            logging.debug('Skipping persistence of enriched secrets object as secrets verification is off,'
                          ' enabled it via env var CKV_VALIDATE_SECRETS and provide an api key')
            return None

        if not self.s3_client:
            logging.error("S3 upload was not correctly initialized")
            return None

        base_path = re.sub(REPO_PATH_PATTERN, r'original_secrets/\1', self.repo_path)
        s3_path = f'{base_path}/{uuid.uuid4()}.json'
        try:
            _put_json_object(self.s3_client, enriched_secrets, self.bucket, s3_path, log_stack_trace_on_error=False)
        except ClientError:
            logging.warning("Got access denied, retrying as s3 role changes should be propagated")
            sleep(4)
            try:
                _put_json_object(self.s3_client, enriched_secrets, self.bucket, s3_path, log_stack_trace_on_error=False)
            except ClientError:
                logging.error("Getting access denied consistently, skipping secrets verification, please try again")
                return None

        return s3_path

    def persist_run_metadata(self, run_metadata: dict[str, str | list[str]]) -> None:
        if not self.use_s3_integration or not self.s3_client or self.s3_setup_failed:
            return
        if not self.bucket or not self.repo_path:
            logging.error(f"Something went wrong: bucket {self.bucket}, repo path {self.repo_path}")
            return
        persist_run_metadata(run_metadata, self.s3_client, self.bucket, self.repo_path, True)
        if self.support_bucket and self.support_repo_path:
            logging.debug(f'Also uploading run_metadata.json to support location: {self.support_bucket}/{self.support_repo_path}')
            persist_run_metadata(run_metadata, self.s3_client, self.support_bucket, self.support_repo_path, False)

    def persist_logs_stream(self, logs_stream: StringIO) -> None:
        if not self.use_s3_integration or not self.s3_client or self.s3_setup_failed:
            return
        if not self.support_bucket or not self.support_repo_path:
            logging.error(
                f"Something went wrong with the log upload location: bucket {self.support_bucket}, repo path {self.support_repo_path}")
            return
        persist_logs_stream(logs_stream, self.s3_client, self.support_bucket, self.support_repo_path)

    def persist_graphs(self, graphs: dict[str, list[tuple[LibraryGraph, Optional[str]]]], absolute_root_folder: str = '') -> None:
        if not self.use_s3_integration or not self.s3_client or self.s3_setup_failed:
            return
        if not self.bucket or not self.repo_path:
            logging.error(f"Something went wrong: bucket {self.bucket}, repo path {self.repo_path}")
            return
        persist_graphs(graphs, self.s3_client, self.bucket, self.repo_path, self.persist_graphs_timeout,
                       absolute_root_folder=absolute_root_folder)

    def persist_resource_subgraph_maps(self, resource_subgraph_maps: dict[str, dict[str, str]]) -> None:
        if not self.use_s3_integration or not self.s3_client or self.s3_setup_failed:
            return
        if not self.bucket or not self.repo_path:
            logging.error(f"Something went wrong: bucket {self.bucket}, repo path {self.repo_path}")
            return
        persist_resource_subgraph_maps(resource_subgraph_maps, self.s3_client, self.bucket, self.repo_path, self.persist_graphs_timeout)

    def commit_repository(self, branch: str) -> str | None:
        """
        :param branch: branch to be persisted
        Finalize the repository's scanning in bridgecrew's platform.
        """
        try_num = 0
        while try_num < MAX_RETRIES:
            if not self.use_s3_integration or self.s3_setup_failed:
                return None

            request = None
            response = None
            try:
                if not self.http:
                    logging.error("HTTP manager was not correctly created")
                    return None
                if not self.bc_source:
                    logging.error("Source was not set")
                    return None
                if not self.bc_source.upload_results:
                    # no need to upload something
                    return None

                logging.debug(f'Submitting finalize upload request to {self.integrations_api_url}')
                request = self.http.request("PUT", f"{self.integrations_api_url}?source={self.bc_source.name}",  # type:ignore[no-untyped-call]
                                            body=json.dumps(
                                                {"path": self.repo_path, "branch": branch,
                                                 "to_branch": CI_METADATA_EXTRACTOR.to_branch,
                                                 "pr_id": CI_METADATA_EXTRACTOR.pr_id,
                                                 "pr_url": CI_METADATA_EXTRACTOR.pr_url,
                                                 "commit_hash": CI_METADATA_EXTRACTOR.commit_hash,
                                                 "commit_url": CI_METADATA_EXTRACTOR.commit_url,
                                                 "author": CI_METADATA_EXTRACTOR.author_name,
                                                 "author_url": CI_METADATA_EXTRACTOR.author_url,
                                                 "run_id": CI_METADATA_EXTRACTOR.run_id,
                                                 "run_url": CI_METADATA_EXTRACTOR.run_url,
                                                 "repository_url": CI_METADATA_EXTRACTOR.repository_url}),
                                            headers=merge_dicts({"Authorization": self.get_auth_token(),
                                                                 "Content-Type": "application/json",
                                                                 'x-api-client': self.bc_source.name,
                                                                 'x-api-checkov-version': checkov_version},
                                                                get_user_agent_header()
                                                                ))
                response = json.loads(request.data.decode("utf8"))
                logging.debug(f'Request ID: {request.headers.get("x-amzn-requestid")}')
                logging.debug(f'Trace ID: {request.headers.get("x-amzn-trace-id")}')
                url: str = self.get_sso_prismacloud_url(response.get("url", None))
                return url
            except HTTPError:
                logging.error(f"Failed to commit repository {self.repo_path}", exc_info=True)
                self.s3_setup_failed = True
            except JSONDecodeError:
                if request:
                    logging.warning(f"Response (status: {request.status}) of {self.integrations_api_url}: {request.data.decode('utf8')}")  # danger:ignore - we won't be here if the response contains valid data
                logging.error(f"Response of {self.integrations_api_url} is not a valid JSON", exc_info=True)
                self.s3_setup_failed = True
            finally:
                if request and request.status == 201 and response and response.get("result") == "Success":
                    logging.info(f"Finalize repository {self.repo_id} in the platform")
                elif (
                    response
                    and try_num < MAX_RETRIES
                    and re.match("The integration ID .* in progress", response.get("message", ""))
                ):
                    logging.info(
                        f"Failed to persist for repo {self.repo_id}, sleeping for {SLEEP_SECONDS} seconds before retrying")
                    try_num += 1
                    sleep(SLEEP_SECONDS)
                else:
                    logging.error(f"Failed to finalize repository {self.repo_id} in the platform with the following error:\n{response}")
                    self.s3_setup_failed = True

        return None

    def persist_files(self, files_to_persist: list[FileToPersist]) -> None:
        logging.info(f"Persisting {len(files_to_persist)} files")
        with futures.ThreadPoolExecutor() as executor:
            futures.wait(
                [executor.submit(self._persist_file, file_to_persist.full_file_path, file_to_persist.s3_file_key) for
                 file_to_persist in files_to_persist],
                return_when=futures.FIRST_EXCEPTION,
            )
        logging.info(f"Done persisting {len(files_to_persist)} files")

    def _persist_file(self, full_file_path: str, s3_file_key: str) -> None:
        tries = MAX_RETRIES
        curr_try = 0

        if not self.s3_client or not self.bucket or not self.repo_path:
            logging.error(
                f"Something went wrong: S3 client {self.s3_client} bucket {self.bucket}, repo path {self.repo_path}"
            )
            return

        file_object_key = os.path.join(self.repo_path, s3_file_key).replace("\\", "/")
        while curr_try < tries:
            try:
                self.s3_client.upload_file(full_file_path, self.bucket, file_object_key)
                return
            except ClientError as e:
                if e.response.get('Error', {}).get('Code') == 'AccessDenied':
                    sleep(SLEEP_SECONDS)
                    curr_try += 1
                else:
                    logging.error(f"failed to persist file {full_file_path} into S3 bucket {self.bucket}",
                                  exc_info=True)
                    raise
            except Exception:
                logging.error(f"failed to persist file {full_file_path} into S3 bucket {self.bucket}", exc_info=True)
                raise
        if curr_try == tries:
            logging.error(
                f"failed to persist file {full_file_path} into S3 bucket {self.bucket} - gut AccessDenied {tries} times")

    def get_platform_run_config(self) -> None:
        if self.skip_download is True:
            logging.debug("Skipping downloading configs from platform")
            return

        if self.is_integration_configured():
            self.get_customer_run_config()
        else:
            self.get_public_run_config()

    def _get_run_config_query_params(self) -> str:
        return f'module={"bc" if self.is_bc_token(self.bc_api_key) else "pc"}&enforcementv2=true'

    def get_run_config_url(self) -> str:
        return f'{self.platform_run_config_url}?{self._get_run_config_query_params()}'

    def get_customer_run_config(self) -> None:
        if self.skip_download is True:
            logging.debug("Skipping customer run config API call")
            return

        if not self.bc_api_key or not self.is_integration_configured():
            raise Exception(
                "Tried to get customer run config, but the API key was missing or the integration was not set up")

        if not self.bc_source:
            logging.error("Source was not set")
            return

        try:
            token = self.get_auth_token()
            headers = merge_dicts(get_auth_header(token),
                                  get_default_get_headers(self.bc_source, self.bc_source_version))

            self.setup_http_manager()
            if not self.http:
                logging.error("HTTP manager was not correctly created")
                return

            platform_type = PRISMA_PLATFORM if self.is_prisma_integration() else BRIDGECREW_PLATFORM

            url = self.get_run_config_url()
            logging.debug(f'Platform run config URL: {url}')
            request = self.http.request("GET", url, headers=headers)  # type:ignore[no-untyped-call]
            request_id = request.headers.get("x-amzn-requestid")
            trace_id = request.headers.get("x-amzn-trace-id")
            logging.debug(f'Request ID: {request_id}')
            logging.debug(f'Trace ID: {trace_id}')
            if request.status == 500:
                error_message = 'An unexpected backend error occurred getting the run configuration from the platform (status code 500). ' \
                                'please contact support and provide debug logs and the values below. You may be able to use the --skip-download option ' \
                                'to bypass this error, but this will prevent platform configurations (e.g., custom policies, suppressions) from ' \
                                f'being used in the scan.\nRequest ID: {request_id}\nTrace ID: {trace_id}'
                logging.error(error_message)
                raise Exception(error_message)
            elif request.status != 200:
                error_message = get_auth_error_message(request.status, self.is_prisma_integration(), False)
                logging.error(error_message)
                raise BridgecrewAuthError(error_message)
            self.customer_run_config_response = json.loads(request.data.decode("utf8"))

            logging.debug(f"Got customer run config from {platform_type} platform")
        except Exception:
            logging.warning(f"Failed to get the customer run config from {self.platform_run_config_url}", exc_info=True)
            raise

    def get_reachability_run_config(self) -> Union[Dict[str, Any], None]:
        if self.skip_download is True:
            logging.debug("Skipping customer run config API call")
            return None

        if not self.bc_api_key or not self.is_integration_configured():
            raise Exception(
                "Tried to get customer run config, but the API key was missing or the integration was not set up")

        if not self.bc_source:
            logging.error("Source was not set")
            return None

        try:
            token = self.get_auth_token()
            headers = merge_dicts(get_auth_header(token),
                                  get_default_get_headers(self.bc_source, self.bc_source_version))

            self.setup_http_manager()
            if not self.http:
                logging.error("HTTP manager was not correctly created")
                return None

            platform_type = PRISMA_PLATFORM if self.is_prisma_integration() else BRIDGECREW_PLATFORM

            request = self.http.request("GET", self.reachability_run_config_url,
                                        headers=headers)  # type:ignore[no-untyped-call]
            if request.status != 200:
                error_message = get_auth_error_message(request.status, self.is_prisma_integration(), False)
                logging.error(error_message)
                raise BridgecrewAuthError(error_message)

            logging.debug(f"Got reachability run config from {platform_type} platform")

            res: Dict[str, Any] = json.loads(request.data.decode("utf8"))
            return res
        except Exception:
            logging.warning(f"Failed to get the reachability run config from {self.reachability_run_config_url}",
                            exc_info=True)
            raise

    def get_runtime_run_config(self) -> None:
        try:
            if self.skip_download is True:
                logging.debug("Skipping customer run config API call")
                raise

            if not self.bc_api_key or not self.is_integration_configured():
                raise Exception(
                    "Tried to get customer run config, but the API key was missing or the integration was not set up")

            if not self.bc_source:
                logging.error("Source was not set")
                raise

            token = self.get_auth_token()
            headers = merge_dicts(get_auth_header(token),
                                  get_default_get_headers(self.bc_source, self.bc_source_version))

            self.setup_http_manager()
            if not self.http:
                logging.error("HTTP manager was not correctly created")
                raise

            platform_type = PRISMA_PLATFORM if self.is_prisma_integration() else BRIDGECREW_PLATFORM
            url = f"{self.runtime_run_config_url}?repoId={self.repo_id}"
            request = self.http.request("GET", url,
                                        headers=headers)  # type:ignore[no-untyped-call]
            if request.status != 200:
                error_message = get_auth_error_message(request.status, self.is_prisma_integration(), False)
                logging.error(error_message)
                raise BridgecrewAuthError(error_message)

            logging.debug(f"Got run config from {platform_type} platform")

            self.runtime_run_config_response = json.loads(request.data.decode("utf8"))
        except Exception:
            logging.debug('could not get runtime info for this repo')

    def get_prisma_build_policies(self, policy_filter: str) -> None:
        """
        Get Prisma policy for enriching runConfig with metadata
        Filters: https://prisma.pan.dev/api/cloud/cspm/policy#operation/get-policy-filters-and-options
        :param policy_filter: comma separated filter string. Example, policy.label=A,cloud.type=aws
        :return:
        """
        if self.skip_download is True:
            logging.debug("Skipping prisma policy API call")
            return
        if not policy_filter:
            return
        if not self.is_prisma_integration():
            return
        if not self.bc_api_key or not self.is_integration_configured():
            raise Exception(
                "Tried to get prisma build policy metadata, "
                "but the API key was missing or the integration was not set up")

        request = None

        try:
            token = self.get_auth_token()
            headers = merge_dicts(get_prisma_auth_header(token), get_prisma_get_headers())

            self.setup_http_manager()
            if not self.http:
                logging.error("HTTP manager was not correctly created")
                return

            logging.debug(f'Prisma policy URL: {self.prisma_policies_url}')
            query_params = convert_prisma_policy_filter_to_dict(policy_filter)
            if self.is_valid_policy_filter(query_params, valid_filters=self.get_prisma_policy_filters()):
                # If enabled and subtype are not explicitly set, use the only acceptable values.
                query_params['policy.enabled'] = True
                query_params['policy.subtype'] = 'build'
                request = self.http.request(  # type:ignore[no-untyped-call]
                    "GET",
                    self.prisma_policies_url,
                    headers=headers,
                    fields=query_params,
                )
                self.prisma_policies_response = json.loads(request.data.decode("utf8"))
                logging.debug("Got Prisma build policy metadata")
            else:
                logging.warning("Skipping get prisma build policies. --policy-metadata-filter will not be applied.")
        except Exception:
            response_message = f': {request.status} - {request.reason}' if request else ''
            logging.warning(
                f"Failed to get prisma build policy metadata from {self.prisma_policies_url}{response_message}", exc_info=True)

    def get_prisma_policy_filters(self) -> Dict[str, Dict[str, Any]]:
        request = None
        try:
            token = self.get_auth_token()
            headers = merge_dicts(get_prisma_auth_header(token), get_prisma_get_headers())

            self.setup_http_manager()
            if not self.http:
                logging.error("HTTP manager was not correctly created")
                return {}

            logging.debug(f'Prisma filter URL: {self.prisma_policy_filters_url}')
            request = self.http.request(  # type:ignore[no-untyped-call]
                "GET",
                self.prisma_policy_filters_url,
                headers=headers,
            )
            policy_filters: dict[str, dict[str, Any]] = json.loads(request.data.decode("utf8"))
            logging.debug(f'Prisma filter suggestion response: {policy_filters}')
            return policy_filters
        except Exception:
            response_message = f': {request.status} - {request.reason}' if request else ''
            logging.warning(
                f"Failed to get prisma build policy metadata from {self.prisma_policy_filters_url}{response_message}", exc_info=True)
            return {}

    @staticmethod
    def is_valid_policy_filter(policy_filter: dict[str, str], valid_filters: dict[str, dict[str, Any]] | None = None) -> bool:
        """
        Validates only the filter names
        """
        valid_filters = valid_filters or {}

        if not policy_filter:
            return False
        if not valid_filters:
            return False
        for filter_name, filter_value in policy_filter.items():
            if filter_name not in valid_filters.keys():
                logging.warning(f"Invalid filter name: {filter_name}")
                logging.warning(f"Available filter names: {', '.join(valid_filters.keys())}")
                return False
            elif filter_name == 'policy.subtype' and filter_value != 'build':
                logging.warning(f"Filter value not allowed: {filter_value}")
                logging.warning("Available options: build")
                return False
            elif filter_name == 'policy.enabled' and not convert_str_to_bool(filter_value):
                logging.warning(f"Filter value not allowed: {filter_value}")
                logging.warning("Available options: True")
                return False
        logging.debug("--policy-metadata-filter is valid")
        return True

    def get_public_run_config(self) -> None:
        if self.skip_download is True:
            logging.debug("Skipping checkov mapping and guidelines API call")
            return
        try:
            headers: dict[str, Any] = {}

            self.setup_http_manager()
            if not self.http:
                logging.error("HTTP manager was not correctly created")
                return

            request = self.http.request("GET", self.guidelines_api_url, headers=headers)  # type:ignore[no-untyped-call]
            if request.status >= 300:
                request = self.http.request(  # type:ignore[no-untyped-call]
                    "GET",
                    self.guidelines_api_url_backoff,
                    headers=headers,
                )

            self.public_metadata_response = json.loads(request.data.decode("utf8"))
            platform_type = PRISMA_PLATFORM if self.is_prisma_integration() else BRIDGECREW_PLATFORM
            logging.debug(f"Got checkov mappings and guidelines from {platform_type} platform")
        except Exception:
            logging.warning(f"Failed to get the checkov mappings and guidelines from {self.guidelines_api_url}. Skips using BC_* IDs will not work.",
                            exc_info=True)

    def get_report_to_platform(self, args: argparse.Namespace, scan_reports: list[Report]) -> None:
        if self.bc_api_key:

            if args.directory:
                repo_id = self.get_repository(args)
                self.setup_bridgecrew_credentials(repo_id=repo_id)
            if self.is_integration_configured():
                self._upload_run(args, scan_reports)

    # Added this to generate a default repo_id for cli scans for upload to the platform
    # whilst also persisting a cli repo_id into the object
    def persist_bc_api_key(self, args: argparse.Namespace) -> str | None:
        if args.bc_api_key:
            self.bc_api_key = args.bc_api_key
        else:
            # get the key from file
            self.bc_api_key = read_key()
        return self.bc_api_key

    # Added this to generate a default repo_id for cli scans for upload to the platform
    # whilst also persisting a cli repo_id into the object
    def persist_repo_id(self, args: argparse.Namespace) -> str:
        if args.repo_id is None:
            if CI_METADATA_EXTRACTOR.from_branch:
                self.repo_id = CI_METADATA_EXTRACTOR.from_branch
            if args.directory:
                basename = path.basename(os.path.abspath(args.directory[0]))
                self.repo_id = f"cli_repo/{basename}"
            if args.file:
                # Get the base path of the file based on it's absolute path
                basename = os.path.basename(os.path.dirname(os.path.abspath(args.file[0])))
                self.repo_id = f"cli_repo/{basename}"

        else:
            self.repo_id = args.repo_id

        if not self.repo_id:
            # this should not happen
            self.repo_id = "cli_repo/unknown"

        return self.repo_id

    def get_repository(self, args: argparse.Namespace) -> str:
        if CI_METADATA_EXTRACTOR.from_branch:
            return CI_METADATA_EXTRACTOR.from_branch
        arg_dir = args.directory[0]
        arg_dir.rstrip(os.path.sep)  # If directory ends with /, remove it. Does not remove any other character!!
        basename = 'unnamed_repo' if path.basename(arg_dir) == '.' else path.basename(arg_dir)
        repo_id = f"cli_repo/{basename}"
        return repo_id

    def _upload_run(self, args: argparse.Namespace, scan_reports: list[Report]) -> None:
        print(Style.BRIGHT + colored("Connecting to Bridgecrew.cloud...", 'green',
                                     attrs=['bold']) + Style.RESET_ALL)
        self.persist_repository(args.directory[0])
        print(Style.BRIGHT + colored("Metadata upload complete", 'green',
                                     attrs=['bold']) + Style.RESET_ALL)
        self.persist_scan_results(scan_reports)
        self.persist_sast_scan_results(scan_reports)
        self.persist_cdk_scan_results(scan_reports)
        print(Style.BRIGHT + colored("Report upload complete", 'green',
                                     attrs=['bold']) + Style.RESET_ALL)
        self.commit_repository(args.branch)
        print(Style.BRIGHT + colored(
            "COMPLETE! \nYour results are in your Bridgecrew dashboard, available here: https://bridgecrew.cloud \n",
            'green', attrs=['bold']) + Style.RESET_ALL)

    def _input_orgname(self) -> str:
        while True:
            result = str(input('Organization name: ')).lower().strip()  # nosec
            # remove spaces and special characters
            result = ''.join(e for e in result if e.isalnum())
            if result:
                break
        return result

    def _input_visualize_results(self) -> str:
        while True:
            result = str(input('Visualize results? (y/n): ')).lower().strip()  # nosec
            if result[:1] in ["y", "n"]:
                break
        return result

    def _input_levelup_results(self) -> str:
        while True:
            result = str(input('Level up? (y/n): ')).lower().strip()  # nosec
            if result[:1] in ["y", "n"]:
                break
        return result

    def _input_email(self) -> str:
        while True:
            email = str(input('E-Mail: ')).lower().strip()  # nosec
            if re.search(EMAIL_PATTERN, email):
                break
            else:
                print("email should match the following pattern: {}".format(EMAIL_PATTERN))
        return email

    @staticmethod
    def loading_output(msg: str) -> None:
        with trange(ACCOUNT_CREATION_TIME) as t:
            for _ in t:
                t.set_description(msg)
                t.set_postfix(refresh=False)
                sleep(SLEEP_SECONDS)

    def repo_matches(self, repo_name: str) -> bool:
        # matches xyz_org/repo or org/repo (where xyz is the BC org name and the CLI repo prefix from the platform)
        return re.match(re.compile(f'^(\\w+_)?{self.repo_id}$'), repo_name) is not None

    def get_default_headers(self, request_type: str) -> dict[str, Any]:
        if not self.bc_source:
            logging.warning("Source was not set")
            return {}

        if request_type.upper() == "GET":
            return merge_dicts(get_default_get_headers(self.bc_source, self.bc_source_version),
                               {"Authorization": self.get_auth_token()})
        elif request_type.upper() == "POST":
            return merge_dicts(get_default_post_headers(self.bc_source, self.bc_source_version),
                               {"Authorization": self.get_auth_token()})

        logging.info(f"Unsupported request {request_type}")
        return {}

    # Define the function that will get the relay state from the Prisma Cloud Platform.
    def get_sso_prismacloud_url(self, report_url: str) -> str:
        if not bc_integration.prisma_api_url or not self.http or not self.bc_source or report_url is None:
            return report_url or ''
        url_saml_config = f"{bc_integration.prisma_api_url}/saml/config"
        token = self.get_auth_token()
        headers = merge_dicts(get_auth_header(token),
                              get_default_get_headers(self.bc_source, self.bc_source_version))

        request = self.http.request("GET", url_saml_config, headers=headers, timeout=10)  # type:ignore[no-untyped-call]
        if request.status >= 300:
            return report_url

        data = json.loads(request.data.decode("utf8"))

        relay_state_param_name = data.get("relayStateParamName")
        access_saml_url = data.get("redLockAccessSamlUrl")

        if relay_state_param_name and access_saml_url:
            parsed_url = urlparse(report_url)
            uri = parsed_url.path
            # If there are any query parameters, append them to the URI
            if parsed_url.query:
                uri = f"{uri}?{parsed_url.query}"
            # Check if the URL already contains GET parameters.
            if "?" in access_saml_url:
                report_url = f"{access_saml_url}&{relay_state_param_name}={uri}"
            else:
                report_url = f"{access_saml_url}?{relay_state_param_name}={uri}"

        return report_url

    def setup_on_prem(self) -> None:
        if self.customer_run_config_response:
            self.on_prem = self.customer_run_config_response.get('tenantConfig', {}).get('preventCodeUploads', False)
            if self.on_prem:
                logging.debug('On prem mode is enabled')


bc_integration = BcPlatformIntegration()



from __future__ import annotations

from typing import Any

from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class DatasyncLocationExposesSecrets(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure DataSync Location Object Storage doesn't expose secrets"
        id = "CKV_AWS_295"
        supported_resources = ("aws_datasync_location_object_storage",)
        categories = (CheckCategories.SECRETS,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "secret_key"

    def get_forbidden_values(self) -> list[Any]:
        return [ANY_VALUE]


check = DatasyncLocationExposesSecrets()



from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.checks.object_registry import Registry as BaseCheckRegistry


class Registry(BaseCheckRegistry):
    def __init__(self) -> None:
        super().__init__(CheckType.OPENAPI)

    def get_key(self, entity_type: str, entity_name: str, check_id: str, file_path: str) -> str:
        return f'{file_path}.{entity_name}.{check_id}'



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class PolicyLoggingEnabled(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure logging at session end is enabled within security policies"
        id = "CKV_PAN_10"
        supported_resources = ('panos_security_policy', 'panos_security_rule_group')
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        # Check there is a rule defined in the resource
        if 'rule' in conf:

            # Report the area of evaluation
            self.evaluated_keys = ['rule']

            # Get all the rules defined in the resource
            rules = conf['rule']

            # Iterate over each rule
            for secrule in rules:

                # Check if logging at session end is defined in the resource
                if 'log_end' in secrule:

                    # If logging at session end is defined, get the value
                    logstatus = secrule['log_end']

                    # Setting log_end to false is a fail, logging will be disabled
                    if not logstatus[0]:
                        return CheckResult.FAILED
                    # The other value for log_end is true, which is a pass

            # If no fails were found in the rules, this is a pass
            # Also, if no log_end attributes were explicitly set, the default is true, which is also a pass
            return CheckResult.PASSED

        # If there's no rules we have nothing to check
        return CheckResult.UNKNOWN


check = PolicyLoggingEnabled()



from typing import Any

from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class ElastiCacheHasCustomSubnet(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        NIST.800-53.r5 AC-4, NIST.800-53.r5 AC-4(21), NIST.800-53.r5 SC-7, NIST.800-53.r5 SC-7(11),
        NIST.800-53.r5 SC-7(16), NIST.800-53.r5 SC-7(21), NIST.800-53.r5 SC-7(4), NIST.800-53.r5 SC-7(5)
        ElastiCache clusters should not use the default subnet group
        """
        name = "Ensure ElastiCache clusters do not use the default subnet group"
        id = "CKV_AWS_323"
        supported_resources = ("aws_elasticache_cluster",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "subnet_group_name"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = ElastiCacheHasCustomSubnet()



from __future__ import annotations

from checkov.kubernetes.graph_builder.graph_components.blocks import KubernetesBlock
from checkov.kubernetes.graph_builder.graph_components.edge_builders.K8SEdgeBuilder import K8SEdgeBuilder


class VertexConncetions:
    def __init__(self, origin_vertex_index: int, destination_vertices_indices: list[int] | None = None) -> None:
        self.origin_vertex_index = origin_vertex_index
        self.destination_vertices_indices = destination_vertices_indices or []


class ServiceAccountEdgeBuilder(K8SEdgeBuilder):
    def __init__(self) -> None:
        self._cache: dict[str, VertexConncetions] = {}

    @staticmethod
    def should_search_for_edges(vertex: KubernetesBlock) -> bool:
        kind: str | None = vertex.attributes.get('kind')
        return kind == 'ServiceAccount'

    def _find_all_service_accounts(self, vertices: list[KubernetesBlock]) -> bool:
        found_service_accounts = False
        for index, vertex in enumerate(vertices):
            service_account_name = vertex.attributes.get('metadata.name')
            if vertex.attributes.get('kind') != 'ServiceAccount' or service_account_name is None:
                continue
            self._cache[service_account_name] = VertexConncetions(index)
            found_service_accounts = True
        return found_service_accounts

    @staticmethod
    def find_connections(vertex: KubernetesBlock, vertices: list[KubernetesBlock]) -> list[int]:
        # DEPRECATED - this is just here for this builder to support the interface. Use `find_connections_for_instance`
        raise NotImplementedError

    def find_connections_for_instance(self, vertex: KubernetesBlock, vertices: list[KubernetesBlock]) -> list[int]:
        if not self._cache:
            found_service_accounts = self._find_all_service_accounts(vertices)
            if found_service_accounts:
                for index, destination_vertex in enumerate(vertices):
                    if destination_vertex.id == vertex.id:
                        continue

                    destination_vertex_ref = destination_vertex.attributes.get('spec.serviceAccountName')
                    if destination_vertex_ref in self._cache:
                        self._cache[destination_vertex_ref].destination_vertices_indices.append(index)

        vertex_ref = vertex.attributes.get('metadata.name')
        if vertex_ref is None:
            return []
        return self._cache[vertex_ref].destination_vertices_indices



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class AppServiceDisallowCORS(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure that CORS disallows every resource to access app services"
        id = "CKV_AZURE_57"
        supported_resources = ('azurerm_app_service', 'azurerm_linux_web_app', 'azurerm_windows_web_app')
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_attribute_result=CheckResult.PASSED)

    def get_inspected_key(self):
        return 'site_config/[0]/cors/[0]/allowed_origins'

    def get_forbidden_values(self):
        return [['*']]


check = AppServiceDisallowCORS()



from __future__ import annotations

import logging
from typing import Any

import jsonschema
from jsonschema import validate


class VCSSchema():
    def __init__(self, schema: dict[str, Any]) -> None:
        self.schema = schema

    def validate(self, data: dict[str, Any] | list[dict[str, Any]]) -> bool:
        try:
            validate(instance=data, schema=self.schema)
        except jsonschema.exceptions.ValidationError:
            logging.debug("validation error", exc_info=True)
            return False
        return True



import json
import os

current_dir = os.path.dirname(os.path.realpath(__file__))


def test_sast_python() -> None:
    report_path = os.path.join(current_dir, '..', 'checkov_report_sast_python.json')
    validate_report(os.path.abspath(report_path))


def test_sast_java() -> None:
    report_path = os.path.join(current_dir, '..', 'checkov_report_sast_java.json')
    validate_report(os.path.abspath(report_path))


def test_sast_javascript() -> None:
    report_path = os.path.join(current_dir, '..', 'checkov_report_sast_javascript.json')
    validate_report(os.path.abspath(report_path))


def test_sast_platform_report_python() -> None:
    report_path = '/tmp/sast_python_report.json'
    validate_platform_report(os.path.abspath(report_path), 'python')


def test_sast_platform_report_java() -> None:
    report_path = '/tmp/sast_java_report.json'
    validate_platform_report(os.path.abspath(report_path), 'java')


def test_sast_platform_report_javascript() -> None:
    report_path = '/tmp/sast_javascript_report.json'
    validate_platform_report(os.path.abspath(report_path), 'javascript')


def validate_report(report_path: str) -> None:
    with open(report_path) as f:
        data = f.read()
        report = json.loads(data)
        assert report is not None
        results = report.get("results")
        assert results is not None
        passed_checks = results.get("passed_checks")
        failed_checks = results.get("failed_checks")
        assert not passed_checks
        assert failed_checks is not None
        assert isinstance(failed_checks, list)
        assert len(failed_checks) > 0
        summary = report.get("summary")
        assert summary.get("passed") == 0
        assert summary.get("failed") > 0


def validate_platform_report(report_path: str, lang: str) -> None:
    with open(report_path) as f:
        data = f.read()
        report = json.loads(data)
        assert report is not None
        rule_matches = report.get("rule_match")
        assert rule_matches is not None
        for check_id, check in rule_matches.get(lang).items():
            assert check.get('check_id') == check_id
            sast_match = check.get('matches')[0]
            assert sast_match['location']['path'] != ""
            assert sast_match['location']['code_block'] != ""



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from checkov.ansible.checks.registry import registry
from checkov.ansible.graph_builder.graph_components.resource_types import ResourceType
from checkov.ansible.graph_builder.local_graph import AnsibleLocalGraph
from checkov.ansible.utils import get_relevant_file_content, build_definitions_context, generate_task_name
from checkov.common.output.report import CheckType
from checkov.common.util.consts import START_LINE, END_LINE
from checkov.yaml_doc.runner import Runner as YamlRunner

if TYPE_CHECKING:
    from checkov.common.checks.base_check_registry import BaseCheckRegistry
    from checkov.common.typing import LibraryGraphConnector
    from checkov.common.runners.graph_builder.local_graph import ObjectLocalGraph
    from checkov.common.runners.graph_manager import ObjectGraphManager
    from collections.abc import Iterable


class Runner(YamlRunner):
    check_type = CheckType.ANSIBLE  # noqa: CCE003  # a static attribute

    def __init__(
        self,
        db_connector: LibraryGraphConnector | None = None,
        source: str = "Ansible",
        graph_class: type[ObjectLocalGraph] = AnsibleLocalGraph,
        graph_manager: ObjectGraphManager | None = None,
    ) -> None:
        super().__init__(
            db_connector=db_connector,
            source=source,
            graph_class=graph_class,
            graph_manager=graph_manager,
        )

    def require_external_checks(self) -> bool:
        return False

    def import_registry(self) -> BaseCheckRegistry:
        return registry

    @staticmethod
    def _parse_file(
        f: str, file_content: str | None = None
    ) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
        content = get_relevant_file_content(file_path=f)
        if content:
            return YamlRunner._parse_file(f=f, file_content=content)

        return None

    def get_resource(
        self,
        file_path: str,
        key: str,
        supported_entities: Iterable[str],
        start_line: int = -1,
        end_line: int = -1,
        graph_resource: bool = False,
    ) -> str:
        if not self.definitions or not isinstance(self.definitions, dict):
            return key

        resource_name = self.generate_resource_name(start_line, end_line, self.definitions[file_path])
        return resource_name if resource_name else key

    def generate_resource_name(
        self,
        start_line: int,
        end_line: int,
        file_conf: dict[str, Any] | list[dict[str, Any]],
        resource_key: str | None = None,
    ) -> str | None:
        if not isinstance(file_conf, list):
            return resource_key

        for code_block in file_conf:
            if code_block[START_LINE] <= start_line <= end_line <= code_block[END_LINE]:
                if ResourceType.TASKS in code_block:
                    for task in code_block[ResourceType.TASKS]:
                        if task[START_LINE] <= start_line <= end_line <= task[END_LINE]:
                            if ResourceType.BLOCK in task:
                                resource_name = self._handle_block_tasks(
                                    start_line=start_line,
                                    end_line=end_line,
                                    code_block=task,
                                )
                                if resource_name is not None:
                                    return resource_name
                            return generate_task_name(task=task) or resource_key
                elif ResourceType.BLOCK in code_block:
                    resource_name = self._handle_block_tasks(
                        start_line=start_line,
                        end_line=end_line,
                        code_block=code_block,
                    )
                    if resource_name is not None:
                        return resource_name
                else:
                    return generate_task_name(task=code_block) or resource_key

        return resource_key

    def _handle_block_tasks(
        self, start_line: int, end_line: int, code_block: dict[str, Any], prefix: str = ""
    ) -> str | None:
        for block_task in code_block[ResourceType.BLOCK]:
            if block_task[START_LINE] <= start_line <= end_line <= block_task[END_LINE]:
                prefix += f"{ResourceType.BLOCK}."  # with each nested level an extra block prefix is added
                if ResourceType.BLOCK in block_task:
                    resource_name = self._handle_block_tasks(
                        start_line=start_line,
                        end_line=end_line,
                        code_block=block_task,
                        prefix=prefix,
                    )
                    if resource_name is not None:
                        return resource_name
                return generate_task_name(task=block_task, prefix=prefix)

        return None

    def build_definitions_context(
        self,
        definitions: dict[str, dict[str, Any] | list[dict[str, Any]]],
        definitions_raw: dict[str, list[tuple[int, str]]],
    ) -> dict[str, dict[str, Any]]:
        return build_definitions_context(definitions=definitions, definitions_raw=definitions_raw)



from __future__ import annotations

import logging
from pathlib import Path

from checkov.cdk.checks_infra.base_registry import BaseCdkRegistry
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.output.report import Report
from checkov.common.sast.consts import CDKLanguages
from checkov.sast.runner import Runner as SastRunner
from checkov.runner_filter import RunnerFilter

logger = logging.getLogger(__name__)

CHECKS_DIR_PATH = Path(__file__).parent / "checks"


class CdkRunner(SastRunner):
    check_type = CheckType.CDK  # noqa: CCE003  # a static attribute

    def __init__(self,) -> None:
        super().__init__()
        self.registry = BaseCdkRegistry(checks_dir=str(CHECKS_DIR_PATH))

    def run(
        self,
        root_folder: str | None = None,
        external_checks_dir: list[str] | None = None,
        files: list[str] | None = None,
        runner_filter: RunnerFilter | None = None,
        collect_skip_comments: bool = True,
    ) -> list[Report]:
        runner_filter = runner_filter or RunnerFilter()
        runner_filter.remove_default_sast_policies = True

        for lang in CDKLanguages.set():
            self.cdk_langs.append(lang)
        reports = super().run(
            root_folder=root_folder,
            external_checks_dir=external_checks_dir,
            files=files,
            runner_filter=runner_filter,
            collect_skip_comments=collect_skip_comments,
        )

        reports_to_keep = []
        for report in reports:
            if report.check_type.startswith('cdk'):
                reports_to_keep.append(report)

        return reports_to_keep



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from checkov.bicep.image_referencer.base_provider import BaseBicepProvider
from checkov.common.util.data_structures_utils import find_in_dict
from checkov.common.util.type_forcers import force_list

if TYPE_CHECKING:
    from networkx import DiGraph


class AzureBicepProvider(BaseBicepProvider):
    def __init__(self, graph_connector: DiGraph) -> None:
        super().__init__(
            graph_connector=graph_connector,
            supported_resource_types=SUPPORTED_AZURE_IMAGE_RESOURCE_TYPES,
        )


def extract_images_from_azurerm_batch_pool(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    containers = find_in_dict(
        input_dict=resource,
        key_path="properties/virtualMachineConfiguration/containerConfiguration/containerImageNames",
    )
    if isinstance(containers, list):
        image_names.extend(container for container in containers if isinstance(container, str))

    return image_names


def extract_images_from_azurerm_container_group(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    properties = resource.get("properties")
    if properties and isinstance(properties, dict):
        containers = properties.get("containers")
        if containers:
            for container in force_list(containers):
                name = find_in_dict(input_dict=container, key_path="properties/image")
                if name and isinstance(name, str):
                    image_names.append(name)
        containers = properties.get("initContainers")
        if containers:
            for container in force_list(containers):
                name = find_in_dict(input_dict=container, key_path="properties/image")
                if name and isinstance(name, str):
                    image_names.append(name)

    return image_names


def extract_images_from_azurerm_web_app(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    containers = find_in_dict(input_dict=resource, key_path="properties/template/containers")
    if containers:
        for container in force_list(containers):
            name = container.get("image")
            if name and isinstance(name, str):
                image_names.append(name)

    return image_names


# needs to be at the bottom to add the defined functions
SUPPORTED_AZURE_IMAGE_RESOURCE_TYPES = {
    "Microsoft.App/containerApps": extract_images_from_azurerm_web_app,
    "Microsoft.Batch/batchAccounts/pools": extract_images_from_azurerm_batch_pool,
    "Microsoft.ContainerInstance/containerGroups": extract_images_from_azurerm_container_group,
    "Microsoft.Web/containerApps": extract_images_from_azurerm_web_app,
}



# flake8: noqa
from termcolor import colored

from checkov.version import version
from checkov.common.version_manager import check_for_update

tool = "Checkov"
banner = r"""
       _               _              
   ___| |__   ___  ___| | _______   __
  / __| '_ \ / _ \/ __| |/ / _ \ \ / /
 | (__| | | |  __/ (__|   < (_) \ V / 
  \___|_| |_|\___|\___|_|\_\___/ \_/  
                                      
By Prisma Cloud | version: {} """.format(version)

new_version = check_for_update("checkov", version)
if new_version:
    banner = (
        "\n"
        + banner
        + "\nUpdate available "
        + colored(version, "grey")
        + " -> "
        + colored(new_version, "green")
        + "\nRun "
        + colored("pip3 install -U checkov", "magenta")
        + " to update \n"
    )



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class ECSServiceFargateLatest(BaseResourceCheck):
    def __init__(self) -> None:
        """
        NIST.800-53.r5 SI-2, NIST.800-53.r5 SI-2(2), NIST.800-53.r5 SI-2(4), NIST.800-53.r5 SI-2(5)
        ECS Fargate services should run on the latest Fargate platform version
        """
        name = "Ensure ECS Fargate services run on the latest Fargate platform version"
        id = "CKV_AWS_332"
        supported_resources = ("aws_ecs_service",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        launch_type = conf.get("launch_type")
        if launch_type and isinstance(launch_type, list) and launch_type[0] == "FARGATE":
            platform_version = conf.get("platform_version")
            if platform_version and isinstance(platform_version, list) and platform_version[0] != "LATEST":
                return CheckResult.FAILED
            return CheckResult.PASSED

        return CheckResult.UNKNOWN


check = ECSServiceFargateLatest()



from __future__ import annotations

import os
import platform
from abc import abstractmethod
from typing import Dict, Optional, Any, Set, TYPE_CHECKING, TypeVar, Generic

import dpath
import igraph
from typing_extensions import TypeAlias  # noqa[TC002]

from checkov.common.checks_infra.registry import get_graph_checks_registry
from checkov.common.graph.checks_infra.registry import BaseRegistry
from checkov.common.graph.graph_builder.consts import GraphSource
from checkov.common.images.image_referencer import ImageReferencerMixin
from checkov.common.models.enums import CheckResult
from checkov.common.output.graph_record import GraphRecord
from checkov.common.output.record import Record
from checkov.common.output.report import Report
from checkov.common.runners.base_runner import BaseRunner
from checkov.common.util.data_structures_utils import pickle_deepcopy
from checkov.common.util.secrets import omit_secret_value_from_graph_checks
from checkov.common.variables.context import EvaluationContext
from checkov.runner_filter import RunnerFilter
from checkov.terraform.modules.module_objects import TFDefinitionKey
from checkov.terraform.checks.data.registry import data_registry
from checkov.terraform.checks.module.registry import module_registry
from checkov.terraform.checks.provider.registry import provider_registry
from checkov.terraform.checks.resource.registry import resource_registry
from checkov.terraform.context_parsers.registry import parser_registry
from checkov.common.graph.graph_builder.graph_components.attribute_names import CustomAttributes
from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph
from checkov.terraform.graph_manager import TerraformGraphManager
from checkov.terraform.image_referencer.manager import TerraformImageReferencerManager
from checkov.terraform.tf_parser import TFParser

if TYPE_CHECKING:
    from networkx import DiGraph
    from checkov.common.checks_infra.registry import Registry
    from checkov.common.images.image_referencer import Image
    from checkov.common.typing import LibraryGraphConnector

_Context = TypeVar("_Context", bound="dict[Any, Any]|None")
_Definitions = TypeVar("_Definitions", bound="dict[Any, Any]|None")
_FilePath = TypeVar("_FilePath")

# Allow the evaluation of empty variables
dpath.options.ALLOW_EMPTY_STRING_KEYS = True


class BaseTerraformRunner(
    ImageReferencerMixin[None],
    BaseRunner[_Definitions, _Context, TerraformGraphManager],
    Generic[_Definitions, _Context, _FilePath],
):
    def __init__(
        self,
        parser: TFParser | None = None,
        db_connector: LibraryGraphConnector | None = None,
        external_registries: list[BaseRegistry] | None = None,
        source: str = GraphSource.TERRAFORM,
        graph_class: type[TerraformLocalGraph] = TerraformLocalGraph,
        graph_manager: TerraformGraphManager | None = None,
    ) -> None:
        super().__init__(file_extensions=[".tf", ".hcl"])
        self.external_registries = [] if external_registries is None else external_registries
        self.graph_class = graph_class
        self.parser = parser or TFParser()
        self.definitions: _Definitions | None = None
        self.context: _Context | None = None
        self.breadcrumbs = None
        self.evaluations_context: Dict[TFDefinitionKey, Dict[str, EvaluationContext]] = {}
        self.graph_manager: TerraformGraphManager = (
            graph_manager
            if graph_manager is not None
            else TerraformGraphManager(
                source=source,
                db_connector=db_connector or self.db_connector,
            )
        )
        self.graph_registry: Registry = get_graph_checks_registry(self.check_type)
        self.definitions_with_modules: dict[str, dict[str, Any]] = {}
        self.referrer_cache: Dict[str, str] = {}
        self.non_referred_cache: Set[str] = set()

    block_type_registries = {  # noqa: CCE003  # a static attribute
        "resource": resource_registry,
        "data": data_registry,
        "provider": provider_registry,
        "module": module_registry,
    }

    @abstractmethod
    def run(
        self,
        root_folder: str | None,
        external_checks_dir: list[str] | None = None,
        files: list[str] | None = None,
        runner_filter: RunnerFilter | None = None,
        collect_skip_comments: bool = True,
    ) -> Report | list[Report]:
        pass

    def load_external_checks(self, external_checks_dir: list[str] | None) -> None:
        if external_checks_dir:
            for directory in external_checks_dir:
                resource_registry.load_external_checks(directory)
                self.graph_registry.load_external_checks(directory)

    def get_connected_node(self, entity: dict[str, Any], root_folder: str) -> Optional[Dict[str, Any]]:
        connected_entity = entity.get("connected_node")
        if not connected_entity:
            return None
        connected_entity_context = self.get_entity_context_and_evaluations(connected_entity)
        if not connected_entity_context:
            return None
        full_file_path = connected_entity[CustomAttributes.FILE_PATH]
        connected_node_data = {}
        connected_node_data["code_block"] = connected_entity_context.get("code_lines")
        connected_node_data["file_path"] = f"{os.sep}{os.path.relpath(full_file_path, root_folder)}"
        connected_node_data["file_line_range"] = [
            connected_entity_context.get("start_line"),
            connected_entity_context.get("end_line"),
        ]
        connected_node_data["resource"] = ".".join(connected_entity_context["definition_path"])
        connected_node_data["entity_tags"] = connected_entity.get("tags", {})
        connected_node_data["evaluations"] = None
        connected_node_data["file_abs_path"] = os.path.abspath(full_file_path)
        connected_node_data["resource_address"] = connected_entity_context.get("address")
        return connected_node_data

    def get_graph_checks_report(
        self, root_folder: str, runner_filter: RunnerFilter, graph: igraph.Graph | None = None
    ) -> Report:
        report = Report(self.check_type)
        checks_results = self.run_graph_checks_results(runner_filter, self.check_type, graph)

        for check, check_results in checks_results.items():
            for check_result in check_results:
                entity = check_result["entity"]
                entity_context = self.get_entity_context_and_evaluations(entity)
                if entity_context:
                    full_file_path = entity[CustomAttributes.FILE_PATH]
                    copy_of_check_result = pickle_deepcopy(check_result)
                    for skipped_check in entity_context.get("skipped_checks", []):
                        if skipped_check["id"] == check.id:
                            copy_of_check_result["result"] = CheckResult.SKIPPED
                            copy_of_check_result["suppress_comment"] = skipped_check["suppress_comment"]
                            break
                    copy_of_check_result["entity"] = entity[CustomAttributes.CONFIG]
                    connected_node_data = self.get_connected_node(entity, root_folder)
                    if platform.system() == "Windows":
                        root_folder = os.path.split(full_file_path)[0]
                    resource_id = ".".join(entity_context["definition_path"])
                    resource = resource_id
                    definition_context_file_path = full_file_path
                    if (
                        entity.get(CustomAttributes.TF_RESOURCE_ADDRESS)
                        and entity.get(CustomAttributes.TF_RESOURCE_ADDRESS) != resource_id
                    ):
                        # for plan resources
                        resource = entity[CustomAttributes.TF_RESOURCE_ADDRESS]
                    entity_config = self.get_graph_resource_entity_config(entity)
                    censored_code_lines = omit_secret_value_from_graph_checks(
                        check=check,
                        check_result=check_result,
                        entity_code_lines=entity_context.get("code_lines", []),
                        entity_config=entity_config,
                        resource_attributes_to_omit=runner_filter.resource_attr_to_omit,
                    )
                    record = Record(
                        check_id=check.id,
                        bc_check_id=check.bc_id,
                        check_name=check.name,
                        check_result=copy_of_check_result,
                        code_block=censored_code_lines,
                        file_path=f"{os.sep}{os.path.relpath(full_file_path, root_folder)}",
                        file_line_range=[
                            entity_context.get("start_line", 1),
                            entity_context.get("end_line", 1),
                        ],
                        resource=resource,
                        entity_tags=entity.get("tags", {}),
                        evaluations=None,
                        check_class=check.__class__.__module__,
                        file_abs_path=os.path.abspath(full_file_path),
                        resource_address=entity_context.get("address"),
                        severity=check.severity,
                        bc_category=check.bc_category,
                        benchmarks=check.benchmarks,
                        connected_node=connected_node_data,
                        definition_context_file_path=definition_context_file_path,
                    )
                    if self.breadcrumbs:
                        breadcrumb = self.breadcrumbs.get(record.file_path, {}).get(resource)
                        if breadcrumb:
                            record = GraphRecord(record, breadcrumb)
                    record.set_guideline(check.guideline)
                    report.add_record(record=record)
        return report

    @abstractmethod
    def get_entity_context_and_evaluations(self, entity: dict[str, Any]) -> dict[str, Any] | None:
        pass

    @abstractmethod
    def run_block(
        self,
        entities: list[dict[str, Any]],
        definition_context: _Context,
        full_file_path: _FilePath,
        root_folder: str,
        report: Report,
        scanned_file: str,
        block_type: str,
        runner_filter: RunnerFilter,
        entity_context_path_header: str | None = None,
        module_referrer: str | None = None,
    ) -> None:
        pass

    def extract_images(
        self,
        graph_connector: DiGraph | None = None,
        definitions: dict[str, dict[str, Any] | list[dict[str, Any]]] | None = None,
        definitions_raw: dict[str, list[tuple[int, str]]] | None = None,
    ) -> list[Image]:
        if not graph_connector:
            # should not happen
            return []

        manager = TerraformImageReferencerManager(graph_connector=graph_connector)
        images = manager.extract_images_from_resources()

        return images

    @staticmethod
    def get_graph_resource_entity_config(entity: dict[str, Any]) -> dict[str, Any]:
        context_parser = parser_registry.context_parsers[entity[CustomAttributes.BLOCK_TYPE]]
        entity_config: dict[str, Any] = entity[CustomAttributes.CONFIG]
        definition_path = context_parser.get_entity_definition_path(entity_config)
        for path in definition_path:
            entity_config = entity_config[path]
        return entity_config



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class ElasticsearchDomainHA(BaseResourceCheck):
    def __init__(self) -> None:
        """NIST.800-53.r5 CP-10, NIST.800-53.r5 CP-6(2), NIST.800-53.r5 SC-36, NIST.800-53.r5 SC-5(2),
         NIST.800-53.r5 SI-13(5)
        Elasticsearch domains should be configured with at least three dedicated master nodes"""
        name = "Ensure Elasticsearch domains are configured with at least three dedicated master nodes for HA"
        id = "CKV_AWS_318"
        supported_resources = ("aws_elasticsearch_domain", "aws_opensearch_domain")
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        self.evaluated_keys = ["cluster_config"]
        config = conf.get("cluster_config")
        if config and isinstance(config, list):
            master_count = config[0].get("dedicated_master_count")
            if (
                master_count
                and isinstance(master_count, list)
                and isinstance(master_count[0], int)
                and master_count[0] >= 3
            ):
                zone_awareness = config[0].get("zone_awareness_enabled")
                if zone_awareness and isinstance(zone_awareness, list) and zone_awareness[0]:
                    return CheckResult.PASSED

        return CheckResult.FAILED


check = ElasticsearchDomainHA()



from __future__ import annotations

from typing import Any, Dict, List

from checkov.common.util.consts import START_LINE, END_LINE


def _get_resource_from_code_block(start_line: int, end_line: int, block_to_inspect: dict[str, Any], inspected_key: str | None) -> str | None:
    if block_to_inspect[START_LINE] <= start_line <= end_line <= block_to_inspect[END_LINE]:
        block_name = block_to_inspect.get('displayName',
                                          block_to_inspect.get('name',
                                                               block_to_inspect.get('job',
                                                                                    block_to_inspect.get('stage',
                                                                                                         False))))
        inspected_key = f'{inspected_key}({block_name})' if block_name else inspected_key
        if block_to_inspect[START_LINE] == start_line:
            return inspected_key
        return generate_resource_key_recursive(start_line, end_line, block_to_inspect, resource_key=inspected_key)
    return None


def generate_resource_key_recursive(start_line: int, end_line: int,
                                    file_conf: Dict[str, Any] | List[Dict[str, Any]], resource_key: str | None = None
                                    ) -> str | None:
    if not isinstance(file_conf, dict):
        return resource_key

    for code_block_name, code_block in file_conf.items():
        if isinstance(code_block, dict):
            new_key = f'{resource_key}.{code_block_name}' if resource_key else code_block_name
            resource = _get_resource_from_code_block(start_line, end_line, code_block, new_key)
            if resource:
                return resource
        elif isinstance(code_block, list):
            for index, item in enumerate(code_block):
                if isinstance(item, dict):
                    resource_key_to_inspect = f'{resource_key}.{code_block_name}[{index}]' if resource_key else f'{code_block_name}[{index}]'
                    resource = _get_resource_from_code_block(start_line, end_line, item, resource_key_to_inspect)
                    if resource:
                        return resource
    return resource_key



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult
from checkov.github.base_github_negative_branch_security import NegativeBranchSecurity


class GithubRequireUpdatedBranch(NegativeBranchSecurity):
    def __init__(self) -> None:
        name = "Ensure open git branches are up to date before they can be merged into codebase"
        id = "CKV_GITHUB_20"
        super().__init__(name=name, id=id, missing_attribute_result=CheckResult.FAILED)

    def get_evaluated_keys(self) -> list[str]:
        return ["required_status_checks/strict"]

    def get_forbidden_values(self) -> list[Any]:
        return [False]


check = GithubRequireUpdatedBranch()



"""Module that checks if there is an updated version of a package available."""
from __future__ import annotations

import os
import pickle  # nosec
import re
from collections.abc import Generator, Callable
from typing import Any

import requests
import sys
import time
from datetime import datetime
from functools import wraps
from tempfile import gettempdir


def cache_results(
    function: Callable[[UpdateChecker, str, str], UpdateResult | None]
) -> Callable[[UpdateChecker, str, str], UpdateResult | None]:
    """Return decorated function that caches the results."""

    def save_to_permacache() -> None:
        """Save the in-memory cache data to the permacache.

        There is a race condition here between two processes updating at the
        same time. It's perfectly acceptable to lose and/or corrupt the
        permacache information as each process's in-memory cache will remain
        in-tact.

        """
        update_from_permacache()
        try:
            if filename is None:
                return

            with open(filename, "wb") as fp:
                pickle.dump(cache, fp, pickle.HIGHEST_PROTOCOL)
        except IOError:
            pass  # Ignore permacache saving exceptions

    def update_from_permacache() -> None:
        """Attempt to update newer items from the permacache."""
        try:
            if filename is None:
                return

            with open(filename, "rb") as fp:
                permacache = pickle.load(fp)  # nosec
        except Exception:  # TODO: Handle specific exceptions
            return  # It's okay if it cannot load
        for key, value in permacache.items():
            if key not in cache or value[0] > cache[key][0]:
                cache[key] = value

    cache: dict[tuple[str, str], tuple[float, UpdateResult | None]] = {}
    cache_expire_time = 3600
    try:
        filename = os.path.join(gettempdir(), "update_checker_cache.pkl")
        update_from_permacache()
    except NotImplementedError:
        filename = None

    @wraps(function)
    def wrapped(obj: UpdateChecker, package_name: str, package_version: str, **extra_data: Any) -> UpdateResult | None:
        """Return cached results if available."""
        now = time.time()
        key = (package_name, package_version)
        if not obj._bypass_cache and key in cache:  # Check the in-memory cache
            cache_time, retval = cache[key]
            if now - cache_time < cache_expire_time:
                return retval
        retval = function(obj, package_name, package_version, **extra_data)
        cache[key] = now, retval
        if filename:
            save_to_permacache()
        return retval

    return wrapped


def query_pypi(package: str, include_prereleases: bool) -> dict[str, Any]:
    """Return information about the current version of package."""
    try:
        response = requests.get(f"https://pypi.org/pypi/{package}/json", timeout=1)
    except requests.exceptions.RequestException:
        return {"success": False}
    if response.status_code != 200:
        return {"success": False}
    data = response.json()
    versions = list(data["releases"].keys())
    versions.sort(key=parse_version, reverse=True)

    version = versions[0]
    for tmp_version in versions:
        if include_prereleases or standard_release(tmp_version):
            version = tmp_version
            break

    upload_time = None
    for file_info in data["releases"][version]:
        if file_info["upload_time"]:
            upload_time = file_info["upload_time"]
            break

    return {"success": True, "data": {"upload_time": upload_time, "version": version}}


def standard_release(version: str) -> bool:
    return version.replace(".", "").isdigit()


# This class must be defined before UpdateChecker in order to unpickle objects
# of this type
class UpdateResult:
    """Contains the information for a package that has an update."""

    def __init__(self, package: str, running: str, available: str, release_date: str | None) -> None:
        """Initialize an UpdateResult instance."""
        self.available_version = available
        self.package_name = package
        self.running_version = running
        if release_date:
            self.release_date: datetime | None = datetime.strptime(release_date, "%Y-%m-%dT%H:%M:%S")
        else:
            self.release_date = None

    def __str__(self) -> str:
        """Return a printable UpdateResult string."""
        retval = f"Version {self.running_version} of {self.package_name} is outdated. Version {self.available_version} "
        if self.release_date:
            retval += f"was released {pretty_date(self.release_date)}."
        else:
            retval += "is available."
        return retval


class UpdateChecker:
    """A class to check for package updates."""

    def __init__(self, *, bypass_cache: bool = False) -> None:
        self._bypass_cache = bypass_cache

    @cache_results
    def check(self, package_name: str, package_version: str) -> UpdateResult | None:
        """Return a UpdateResult object if there is a newer version."""

        data = query_pypi(package_name, include_prereleases=not standard_release(package_version))

        if not data.get("success") or (parse_version(package_version) >= parse_version(data["data"]["version"])):
            return None

        return UpdateResult(
            package_name,
            running=package_version,
            available=data["data"]["version"],
            release_date=data["data"]["upload_time"],
        )


def pretty_date(the_datetime: datetime) -> str:
    """Attempt to return a human-readable time delta string."""
    # Source modified from
    # http://stackoverflow.com/a/5164027/176978
    diff = datetime.utcnow() - the_datetime
    if diff.days > 7 or diff.days < 0:
        return the_datetime.strftime("%A %B %d, %Y")
    elif diff.days == 1:
        return "1 day ago"
    elif diff.days > 1:
        return f"{diff.days} days ago"
    elif diff.seconds <= 1:
        return "just now"
    elif diff.seconds < 60:
        return f"{diff.seconds} seconds ago"
    elif diff.seconds < 120:
        return "1 minute ago"
    elif diff.seconds < 3600:
        return f"{int(round(diff.seconds / 60))} minutes ago"
    elif diff.seconds < 7200:
        return "1 hour ago"
    else:
        return f"{int(round(diff.seconds / 3600))} hours ago"


def update_check(package_name: str, package_version: str, bypass_cache: bool = False) -> None:
    """Convenience method that outputs to stderr if an update is available."""
    checker = UpdateChecker(bypass_cache=bypass_cache)
    result = checker.check(package_name, package_version)
    if result:
        print(result, file=sys.stderr)


# The following section of code is taken from setuptools pkg_resources.py (PSF
# license). Unfortunately importing pkg_resources to directly use the
# parse_version function results in some undesired side effects.

component_re = re.compile(r"(\d+ | [a-z]+ | \.| -)", re.VERBOSE)
replace = {"pre": "c", "preview": "c", "-": "final-", "rc": "c", "dev": "@"}.get


def _parse_version_parts(s: str) -> Generator[str, None, None]:
    for part in component_re.split(s):
        part = replace(part, part)
        if not part or part == ".":
            continue
        if part[:1] in "0123456789":
            yield part.zfill(8)  # pad for numeric comparison
        else:
            yield "*" + part

    yield "*final"  # ensure that alpha/beta/candidate are before final


def parse_version(s: str) -> tuple[str, ...]:
    """Convert a version string to a chronologically-sortable key.

    This is a rough cross between distutils' StrictVersion and LooseVersion;
    if you give it versions that would work with StrictVersion, then it behaves
    the same; otherwise it acts like a slightly-smarter LooseVersion. It is
    *possible* to create pathological version coding schemes that will fool
    this parser, but they should be very rare in practice.

    The returned value will be a tuple of strings.  Numeric portions of the
    version are padded to 8 digits so they will compare numerically, but
    without relying on how numbers compare relative to strings.  Dots are
    dropped, but dashes are retained.  Trailing zeros between alpha segments
    or dashes are suppressed, so that e.g. "2.4.0" is considered the same as
    "2.4". Alphanumeric parts are lower-cased.

    The algorithm assumes that strings like "-" and any alpha string that
    alphabetically follows "final"  represents a "patch level".  So, "2.4-1"
    is assumed to be a branch or patch of "2.4", and therefore "2.4.1" is
    considered newer than "2.4-1", which in turn is newer than "2.4".

    Strings like "a", "b", "c", "alpha", "beta", "candidate" and so on (that
    come before "final" alphabetically) are assumed to be pre-release versions,
    so that the version "2.4" is considered newer than "2.4a1".

    Finally, to handle miscellaneous cases, the strings "pre", "preview", and
    "rc" are treated as if they were "c", i.e. as though they were release
    candidates, and therefore are not as new as a version string that does not
    contain them, and "dev" is replaced with an '@' so that it sorts lower than
    than any other pre-release tag.

    """
    parts: list[str] = []
    for part in _parse_version_parts(s.lower()):
        if part.startswith("*"):
            if part < "*final":  # remove '-' before a prerelease tag
                while parts and parts[-1] == "*final-":
                    parts.pop()
            # remove trailing zeros from each series of numeric parts
            while parts and parts[-1] == "00000000":
                parts.pop()
        parts.append(part)
    return tuple(parts)



from __future__ import annotations

from typing import List, Any

from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class LambdaServicePermission(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that AWS Lambda function permissions delegated to AWS services are limited by SourceArn or SourceAccount"
        id = "CKV_AWS_364"
        supported_resources = ("AWS::Lambda::Permission",)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        properties = conf.get('Properties')
        if properties and isinstance(properties, dict):
            principal = properties.get('Principal')
            if principal and isinstance(principal, str):
                principal_parts = principal.split('.')
                try:
                    if principal_parts[1] == 'amazonaws' and principal_parts[2] == 'com':
                        if properties.get('SourceArn') or properties.get('SourceAccount'):
                            return CheckResult.PASSED
                        else:
                            return CheckResult.FAILED
                except IndexError:
                    print("Not a service principal")
                    # Not a service principal, so pass.
                    return CheckResult.UNKNOWN
        return CheckResult.UNKNOWN

    def get_evaluated_keys(self) -> List[str]:
        return ['Properties/Principal', 'Properties/SourceArn', 'Properties/SourceAccount']


check = LambdaServicePermission()



from __future__ import annotations

import re
from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.serverless.checks.function.base_function_check import BaseFunctionCheck
from checkov.common.models.consts import access_key_pattern, secret_key_pattern
from checkov.serverless.parsers.parser import ENVIRONMENT_TOKEN


class AWSCredentials(BaseFunctionCheck):
    def __init__(self) -> None:
        name = "Ensure no hard coded AWS access key and secret key exists in provider"
        id = "CKV_AWS_41"
        supported_entities = ('serverless_aws',)
        categories = (CheckCategories.SECRETS,)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_entities)

    def scan_function_conf(self, conf: dict[str, Any]) -> CheckResult:
        """
        see: https://www.terraform.io/docs/providers/aws/index.html#static-credentials
        """
        result = CheckResult.PASSED
        if conf.get(ENVIRONMENT_TOKEN) and isinstance(conf[ENVIRONMENT_TOKEN], dict):
            env_variables_strings = {key: value for key, value in conf.get(ENVIRONMENT_TOKEN, {}).items() if
                                     isinstance(value, str)}
            for idx, env_var_value in enumerate(env_variables_strings.values()):
                if any([re.match(access_key_pattern, env_var_value), re.match(secret_key_pattern, env_var_value)]):
                    conf[f'{self.id}_secret_{idx}'] = env_var_value
                    result = CheckResult.FAILED
        return result


check = AWSCredentials()



from cdk_integration_tests.utils import run_check
import pytest


@pytest.mark.skip(reason="Typescript not supported yet")
def test_CKV_AWS_18_S3BucketLogging():
    run_check(lang="typescript", check_name="S3BucketLogging")


@pytest.mark.skip(reason="Typescript not supported yet")
def test_CKV_AWS_19_S3BucketEncryption():
    run_check(lang="typescript", check_name="S3BucketEncryption")


@pytest.mark.skip(reason="Typescript not supported yet")
def test_CKV_AWS_21_S3BucketVersioning():
    run_check(lang="typescript", check_name="S3BucketVersioning")


@pytest.mark.skip(reason="Typescript not supported yet")
def test_CKV_AWS_145_S3BucketKMSEncryption():
    run_check(lang="typescript", check_name="S3BucketKMSEncryption")


@pytest.mark.skip(reason="Typescript not supported yet")
def test_CKV2_AWS_6_S3BucketPublicAccessBlock():
    run_check(lang="typescript", check_name="S3BucketPublicAccessBlock")



from __future__ import annotations

from typing import Any

from cloudsplaining.scan.resource_policy_document import ResourcePolicyDocument

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class SNSTopicPolicyAnyPrincipal(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure SNS topic policy is not public by only allowing specific services or principals to access it"
        id = "CKV_AWS_169"
        supported_resources = ("aws_sns_topic_policy",)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        conf_policy = conf.get("policy")
        if conf_policy:
            if isinstance(conf_policy[0], dict):
                try:
                    policy = ResourcePolicyDocument(policy=conf_policy[0])
                    if policy.internet_accessible_actions:
                        return CheckResult.FAILED
                except (TypeError, AttributeError):
                    return CheckResult.UNKNOWN
            else:
                return CheckResult.UNKNOWN
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> list[str]:
        return ["policy"]


check = SNSTopicPolicyAnyPrincipal()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class ElasticCacheAutomaticBackup(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure that Amazon ElastiCache Redis clusters have automatic backup turned on"
        id = "CKV_AWS_134"
        supported_resources = ["aws_elasticache_cluster"]
        categories = [CheckCategories.BACKUP_AND_RECOVERY]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            missing_attribute_result=CheckResult.FAILED,
        )

    def scan_resource_conf(self, conf):
        if conf.get("engine") == ["memcached"]:
            return CheckResult.UNKNOWN

        return super().scan_resource_conf(conf)

    def get_inspected_key(self):
        return "snapshot_retention_limit"

    def get_forbidden_values(self):
        return [0]


check = ElasticCacheAutomaticBackup()



from typing import Dict, Any, Tuple

from checkov.common.checks.base_check_registry import BaseCheckRegistry


class Registry(BaseCheckRegistry):
    def extract_entity_details(self, entity: Dict[str, Any]) -> Tuple[str, str, Dict[str, Any]]:
        provider_type = list(entity.keys())[0]
        provider_name = list(entity.keys())[0]
        provider_configuration = entity[provider_name]
        return provider_type, provider_name, provider_configuration



from __future__ import annotations

from typing import List, Tuple


def get_code_block_from_start(lines: List[str], start: int) -> List[Tuple[int, str]]:
    code_block = [(index, line) for index, line in enumerate(lines, start=start)]
    return cut_code_block_ident(code_block)


def cut_code_block_ident(code_block: List[Tuple[int, str]]) -> List[Tuple[int, str]]:
    min_ident = len(code_block[0][1]) - len(code_block[0][1].lstrip())
    for item in code_block[1:]:
        current_min_ident = len(item[1]) - len(item[1].lstrip())
        if current_min_ident < min_ident:
            min_ident = current_min_ident

    if min_ident == 0:
        return code_block

    code_block_cut_ident = []
    for item in code_block:
        code_block_cut_ident.append((item[0], item[1][min_ident:]))
    return code_block_cut_ident



from __future__ import annotations

from abc import abstractmethod
from collections.abc import Iterable
from typing import TYPE_CHECKING, Any

from checkov.common.checks.base_check import BaseCheck
from checkov.azure_pipelines.checks.registry import registry

if TYPE_CHECKING:
    from checkov.common.models.enums import CheckCategories, CheckResult


class BaseAzurePipelinesCheck(BaseCheck):
    def __init__(
        self,
        name: str,
        id: str,
        categories: Iterable[CheckCategories],
        supported_entities: Iterable[str],
        block_type: str,
        path: str | None = None,
    ) -> None:
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_entities,
            block_type=block_type,
        )
        self.path = path
        registry.register(self)

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:
        self.entity_type = entity_type

        return self.scan_conf(conf)

    @abstractmethod
    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        pass



import re
from typing import Union, Dict, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck

# https://docs.microsoft.com/en-us/azure/templates/microsoft.network/networksecuritygroups
# https://docs.microsoft.com/en-us/azure/templates/microsoft.network/networksecuritygroups/securityrules

INTERNET_ADDRESSES = ["*", "0.0.0.0", "<nw>/0", "/0", "internet", "any"]  # nosec
PORT_RANGE = re.compile(r"\d+-\d+")


class NSGRulePortAccessRestricted(BaseResourceCheck):
    def __init__(self, name: str, check_id: str, port: int) -> None:
        supported_resources = (
            "Microsoft.Network/networkSecurityGroups",
            "Microsoft.Network/networkSecurityGroups/securityRules",
        )
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=check_id, categories=categories, supported_resources=supported_resources)
        self.port = port

    def is_port_in_range(self, port_range: Union[int, str]) -> bool:
        port_range_str = str(port_range)
        if re.match(PORT_RANGE, port_range_str):
            start, end = int(port_range_str.split("-")[0]), int(port_range_str.split("-")[1])
            if start <= self.port <= end:
                return True
        if port_range in (str(self.port), "*"):
            return True
        return False

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        if "properties" in conf:
            securityRules = []
            if self.entity_type == "Microsoft.Network/networkSecurityGroups":
                if "securityRules" in conf["properties"]:
                    securityRules.extend(conf["properties"]["securityRules"])
            if self.entity_type == "Microsoft.Network/networkSecurityGroups/securityRules":
                securityRules.append(conf)

            for rule in securityRules:
                portRanges = []
                sourcePrefixes = []
                if "properties" in rule:
                    if "access" in rule["properties"] and rule["properties"]["access"].lower() == "allow":
                        if "direction" in rule["properties"] and rule["properties"]["direction"].lower() == "inbound":
                            if "protocol" in rule["properties"] and rule["properties"]["protocol"].lower() in ("tcp", "*"):
                                if "destinationPortRanges" in rule["properties"]:
                                    portRanges.extend(rule["properties"]["destinationPortRanges"])
                                if "destinationPortRange" in rule["properties"]:
                                    portRanges.append(rule["properties"]["destinationPortRange"])

                                if "sourceAddressPrefixes" in rule["properties"]:
                                    sourcePrefixes.extend(rule["properties"]["sourceAddressPrefixes"])
                                if "sourceAddressPrefix" in rule["properties"]:
                                    sourcePrefixes.append(rule["properties"]["sourceAddressPrefix"])

                                for portRange in portRanges:
                                    if self.is_port_in_range(portRange):
                                        for prefix in sourcePrefixes:
                                            if prefix in INTERNET_ADDRESSES:
                                                return CheckResult.FAILED

        return CheckResult.PASSED



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class GKEUseCosImage(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Container-Optimized OS (cos) is used for Kubernetes Engine Clusters Node image"
        id = "CKV_GCP_22"
        supported_resources = ['google_container_node_pool']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
        CKV_GCP_22
        This a legacy check, Creation of node pools using node images based on Docker
        container runtimes is not supported in GKE v1.24+ clusters as Dockershim has been removed in Kubernetes v1.24.
        """
        if conf.get('version') and isinstance(conf.get('version'), list):
            raw = conf.get('version')[0]
            splitter = raw.split(".")

            if len(splitter) >= 2:
                version = float(splitter[0] + "." + splitter[1])
                if version >= 1.24:
                    return CheckResult.UNKNOWN

                if 'node_config' in conf:
                    node_config = conf.get('node_config', [{}])[0]
                self.evaluated_keys = ['node_config']
                if not isinstance(node_config, dict):
                    return CheckResult.UNKNOWN

                if conf.get('node_config', [{}])[0].get('image_type', [''])[0].lower().startswith('cos'):
                    self.evaluated_keys = ['node_config/[0]/image_type']
                    return CheckResult.PASSED
                if conf.get('remove_default_node_pool', [{}])[0]:
                    self.evaluated_keys.append('remove_default_node_pool')
                    return CheckResult.PASSED
            return CheckResult.FAILED

        return CheckResult.UNKNOWN


check = GKEUseCosImage()
